# encoding: UTF-8

from opendatatools.aqi2.util import *
import pandas as pd
import time
import json
import hashlib
import base64

from opendatatools.common import RestAgent
from bs4 import BeautifulSoup

class AQIStudyAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    # date format : yyyy-mm-dd
    def get_daily_hour_aqi(self, city, date):
        startTime = "%s 00:00:00" % date
        endTime   = "%s 23:59:59" % date
        return self.get_server_data(city, "HOUR", startTime, endTime)

    def get_hist_daily_aqi(self, city, begindate, enddate):
        startTime = "%s 00:00:00" % begindate
        endTime   = "%s 23:59:59" % enddate
        return self.get_server_data(city, "DAY", startTime, endTime)

    def get_server_data(self, city, type, startTime, endTime):
        url = "https://www.aqistudy.cn/apinew/aqistudyapi.php"
        appid = "1a45f75b824b2dc628d5955356b5ef18"
        method = "GETDETAIL"
        timestamp = int(time.time() * 1000)
        clienttype = "WEB"
        object = {"city": city, "type": type, "startTime" : startTime, "endTime" : endTime }
        secret_key = appid + method + str(timestamp) + clienttype + "{\"city\":\"%s\",\"endTime\":\"%s\",\"startTime\":\"%s\",\"type\":\"%s\"}" % (
            object["city"], object["endTime"], object["startTime"], object["type"]
        )
        secret = hashlib.md5(secret_key.encode("utf8")).hexdigest()
        param = {
            "appId": appid,
            "method": method,
            "timestamp": timestamp,
            "clienttype": clienttype,
            "object": object,
            "secret": secret
        }

        param = base64.standard_b64encode(json.dumps(param).encode("utf8")).decode()
        param = aes_encrypt(real_aes_client_key, real_aes_client_iv, param)

        response = self.do_request(url, param={"d" : param}, method="POST")
        if response is None:
            return None, "获取数据失败"

        #data = base64.standard_b64decode(response.encode("utf8")).decode()
        data = decrypt_response(real_des_key, real_des_iv, real_aes_server_key, real_aes_server_iv, response)

        jsonobj = json.loads(data)
        success = jsonobj["success"]
        errcode = jsonobj["errcode"]
        errmsg  = jsonobj["errmsg"]

        if errcode != 0:
            return None, errmsg

        result = jsonobj["result"]["data"]["rows"]
        df = pd.DataFrame(result)
        if len(df) > 0:
            df.set_index("time", inplace=True)
        return df, ""

    # type : "DAY", "HOUR
    def get_api_map(self, type, timepoint):
        url = "https://www.aqistudy.cn/apinew/aqistudyapi.php"
        appid = "1a45f75b824b2dc628d5955356b5ef18"
        method = "GETMAPDATA"
        timestamp = int(time.time() * 1000)
        clienttype = "WEB"
        object = {"type": type, "timepoint" : timepoint  }
        secret_key = appid + method + str(timestamp) + clienttype + "{\"timepoint\":\"%s\",\"type\":\"%s\"}" % (
            object["timepoint"], object["type"]
        )
        secret = hashlib.md5(secret_key.encode("utf8")).hexdigest()
        param = {
            "appId": appid,
            "method": method,
            "timestamp": timestamp,
            "clienttype": clienttype,
            "object": object,
            "secret": secret
        }

        param = base64.standard_b64encode(json.dumps(param).encode("utf8")).decode()
        param = aes_encrypt(real_aes_client_key, real_aes_client_iv, param)

        response = self.do_request(url, param={"d" : param}, method="POST")
        if response is None:
            return None, "获取数据失败"

        #data = base64.standard_b64decode(response.encode("utf8")).decode()
        data = decrypt_response(real_des_key, real_des_iv, real_aes_server_key, real_aes_server_iv, response)

        jsonobj = json.loads(data)
        success = jsonobj["success"]
        errcode = jsonobj["errcode"]
        errmsg  = jsonobj["errmsg"]

        if errcode != 0:
            return None, errmsg

        result = jsonobj["result"]["data"]["rows"]
        df = pd.DataFrame(result)
        if len(df) > 0:
            df.set_index("time", inplace=True)
        return df, ""

    def get_city_list(self):
        url = "https://www.aqistudy.cn/historydata/"
        response = self.do_request(url, method="GET")

        soup = BeautifulSoup(response, "html5lib")
        links = soup.find_all('a')

        data = {}
        for link in links:
            href = link["href"]
            if href.startswith("monthdata.php?city="):
                city = href.replace("monthdata.php?city=", "")
                data[city] = city
        return data

if __name__ == '__main__':
    aqi = AQIStudyAgent()
    #city = aqi.get_city_list()
    #print(city)

    #df, msg = aqi.get_daily_hour_aqi('北京', '2019-07-22')
    #print(df)
    #print(msg)

    #df, msg = aqi.get_hist_daily_aqi('北京','2018-05-01', '2019-05-01')
    #print(df)
    #print(msg)

    param = "tdgHOYxwKdDSgYXe+RLPzYCgLvrddahasI5XXklB4gVLYqab+XRPpMD/oSqnJ/aEmFwzVEUhLnPzRy03+X1BI7EP40t1A25axHw6XgQEwrMCA8LPhIJdpk3JqlHQwF/0lHvOCRXiksQ2gxlp8zXTAHX2NeaTAHQeAHVjfS2WSo59EEsCNxnpiBc9JH6WSot2V5FpFF1Z0ElMw4F5ts9OERUzyzCtS80roZmhfEN2sQMsQdt+8keJnsk56MDt1HG9THFJI56TYqd+cFcfcjv5UXDQqjBrttIwXzTYLmMVW0ALdQH4MsmIyMsxiHPYMsto6FJWs/7Jh3L3giRN0yVlF8mW73XSkAsiW/aCRcua8psHxI4f5lyG1a2owB5CZS/FMCM3OHeSvmebXVLrPfOeWA=="
    result = aes_decrypt(real_aes_client_key, real_aes_client_iv, param)
    result = base64.standard_b64decode(result).decode()
    print(result)

    #df, msg = aqi.get_real_aqi_map("HOUR", "2019-07-22 08:00:00")
    #print(df)

    df, msg = aqi.get_api_map("DAY", "2019-07-21")
    print(df)



# encoding: utf-8

from opendatatools.common import RestAgent
import pandas as pd
from bs4 import BeautifulSoup
import json

class MovieAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    @staticmethod
    def clear_text(text):
        return text.replace('\n', '').strip()

    def _get_boxoffice_onepage(self, page_no):
        url = 'http://www.cbooo.cn/BoxOffice/getInland'
        data = {
            'pIndex' : page_no,
            't'       : 0,
        }
        response = self.do_request(url, param=data, encoding='utf8')
        if response is None:
            return None

        jsonobj = json.loads(response)

        df = pd.DataFrame(jsonobj)
        return df

    def get_boxoffice(self):
        df_list= []
        for page_no in range(1,6):
            df = self._get_boxoffice_onepage(page_no)
            if df is None:
                return None, '获取数据失败'

            df_list.append(df)

        if len(df_list) == 0:
            return None, '获取数据失败'

        return pd.concat(df_list), ''

    def get_realtime_boxoffice(self):
        url = 'http://www.cbooo.cn/BoxOffice/GetHourBoxOffice'
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'

        jsonobj = json.loads(response)
        return pd.DataFrame(jsonobj['data2']), ''

    def get_recent_boxoffice(self, day):
        url = 'http://www.cbooo.cn/BoxOffice/GetDayBoxOffice?num=-%d' % (day)
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'

        jsonobj = json.loads(response)
        df = pd.DataFrame(jsonobj['data1'])
        df['date'] = jsonobj['data2'][0]['BoxDate']
        return df, ''

    def get_monthly_boxoffice(self, date):
        url = 'http://www.cbooo.cn/BoxOffice/getMonthBox?sdate=%s' % (date)
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'

        jsonobj = json.loads(response)
        return pd.DataFrame(jsonobj['data1']), ''

    def get_yearly_boxoffice(self, year):
        url = 'http://www.cbooo.cn/year?year=%d' % (year)
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'

        soup = BeautifulSoup(response, "html5lib")
        divs = soup.find_all('div')

        data = []
        for div in divs:
            if div.has_attr('id') and div['id'] == 'tabcont':
                rows = div.table.findAll('tr')
                for row in rows:
                    cols = row.findAll('td')
                    if len(cols) == 7:
                        movie_name = MovieAgent.clear_text(cols[0].text)
                        movie_type = MovieAgent.clear_text(cols[1].text)
                        boxoffice  = MovieAgent.clear_text(cols[2].text)
                        avgprice   = MovieAgent.clear_text(cols[3].text)
                        audience   = MovieAgent.clear_text(cols[4].text)
                        country    = MovieAgent.clear_text(cols[5].text)
                        showtime   = MovieAgent.clear_text(cols[6].text)

                        data.append({
                            "movie_name": movie_name,
                            "movie_type": movie_type,
                            "boxoffice" : boxoffice,
                            "avgprice"  : avgprice,
                            "audience"  : audience,
                            "country"   : country,
                            "showtime"  : showtime,
                        })

        return pd.DataFrame(data), ''


# encoding: utf-8

from opendatatools.common import RestAgent
import json
import pandas as pd

class ChinaMoneyAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    # cpr : central parity rate
    def get_hist_cny_cpr(self, start_date, end_date):
        #url = 'http://www.chinamoney.com.cn/dqs/rest/dqs-u-fx-base/CcprHisNew'
        url = 'http://www.chinamoney.com.cn/ags/ms/cm-u-bk-ccpr/CcprHisNew'

        data = {
            'startDate' : start_date,
            'endDate'   : end_date,
        }

        response = self.do_request(url, data)
        rsp = json.loads(response)

        data = rsp['data']
        head = data['head']
        message = data['flagMessage']
        if len(message) > 0:
            return None, message
        else:
            records = rsp['records']
            row_data = []
            for rec in records:
                single_row = []
                single_row.append(rec['date'])
                single_row.extend(rec['values'])
                row_data.append(single_row)

            df = pd.DataFrame(row_data)
            columns = ['date']
            columns.extend(head)
            df.columns = columns
            return df, ""

    def get_cny_spot_price(self):
        url = 'http://www.chinamoney.com.cn/r/cms/www/chinamoney/data/fx/rfx-sp-quot.json'
        response = self.do_request(url, None)
        rsp = json.loads(response)
        time = rsp['data']['showDateCN']
        records = rsp['records']
        df = pd.DataFrame(records)
        df['time'] = time
        return df

    def get_realtime_shibor(self):
        url = 'http://www.chinamoney.com.cn/r/cms/www/chinamoney/data/shibor/shibor.json'
        response = self.do_request(url, None)
        rsp = json.loads(response)
        time = rsp['data']['showDateCN']
        records = rsp['records']
        df = pd.DataFrame(records)
        df['time'] = time
        return df

    def get_his_shibor(self, start_date, end_date):
        url = 'http://www.chinamoney.com.cn/ags/ms/cm-u-bk-shibor/ShiborHis'
        data = {
            'startDate' : start_date,
            'endDate'   : end_date,
        }

        response = self.do_request(url, data)
        rsp = json.loads(response)

        data = rsp['data']
        message = data['message']
        if len(message) > 0:
            return None, message
        else:
            records = rsp['records']
            df = pd.DataFrame(records)
            return df, ""



# encoding: utf-8

from .robo_agent import RoboAgent

robo_agent = RoboAgent()

def set_proxies(proxies):
    robo_agent.set_proxies(proxies)

# 使用前，请在通联数据萝卜投研注册帐号
def login(username, password):
    return robo_agent.login(username, password)

def get_top_items():
    return robo_agent.get_top_items()

def get_sub_items(itemid):
    return robo_agent.get_sub_items(itemid)

def get_series(seriesid):
    return robo_agent.get_series(seriesid)



# encoding: utf-8

from .swindex_agent import SWIndexAgent

sw_agent = SWIndexAgent()

def get_index_list():
    return sw_agent.get_index_list()

def get_index_cons(index_code):
    return sw_agent.get_index_cons(index_code)

def get_index_daily(index_code, start_date, end_date):
    return sw_agent.get_index_daily(index_code, start_date, end_date)

def get_index_dailyindicator(index_code, start_date, end_date, freq):
    if freq == "D":
        type = "Day"
    elif freq == "M":
        type = "Month"
    elif freq == "W":
        type = "Week"
    else:
        type = "Day"
    return sw_agent.get_index_dailyindicator(index_code, start_date, end_date, type)




# encoding: utf-8

from opendatatools.common import RestAgent
from opendatatools.common import date_convert, remove_non_numerical
from bs4 import BeautifulSoup
import datetime
import json, re
import pandas as pd
import io
from opendatatools.futures.futures_agent import _concat_df
import zipfile


def time_map(x):
    if x == '':
        return ''
    else:
        return datetime.datetime.strptime(x, '%Y%m%d').strftime('%Y-%m-%d')


def plan_map(x):
    if '派' not in x:
        return 0
    else:
        return '%.3f' % (float(x.split('派')[-1].split('元')[0]) / 10)


class SHExAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)
        headers = {
            "Accept": '*/*',
            'Referer': 'http://www.sse.com.cn/market/sseindex/indexlist/',
            'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/66.0.3359.181 Safari/537.36',
        }
        self.add_headers(headers)

    def get_index_list(self):
        url = 'http://query.sse.com.cn/commonSoaQuery.do'
        data = {
            'sqlId': 'DB_SZZSLB_ZSLB',
        }

        response = self.do_request(url, data)
        rsp = json.loads(response)

        if 'pageHelp' in rsp:
            data = rsp['pageHelp']['data']
            return pd.DataFrame(data)
        else:
            return None

    def get_index_component(self, index):
        url = 'http://query.sse.com.cn/commonSoaQuery.do'
        data = {
            'sqlId': 'DB_SZZSLB_CFGLB',
            'indexCode': index,
        }

        response = self.do_request(url, data)
        rsp = json.loads(response)

        if 'pageHelp' in rsp:
            data = rsp['pageHelp']['data']
            return pd.DataFrame(data)
        else:
            return None

    def get_dividend(self, code):
        url = 'http://query.sse.com.cn/commonQuery.do'
        data = {
            'sqlId': 'COMMON_SSE_GP_SJTJ_FHSG_AGFH_L_NEW',
            'security_code_a': code,
        }

        response = self.do_request(url, data)
        rsp = json.loads(response)

        if 'result' in rsp:
            data = rsp['result']
            return pd.DataFrame(data)
        else:
            return None

    def get_rzrq_info(self, date):
        """
        融资融券 上交所网址
        http://www.sse.com.cn/market/othersdata/margin/sum/

        下载地址：
        http://www.sse.com.cn/market/dealingdata/overview/margin/a/rzrqjygk20201224.xls
        :param date:
        :return:
        """
        date2 = date_convert(date, '%Y-%m-%d', '%Y%m%d')
        url = 'http://www.sse.com.cn/market/dealingdata/overview/margin/a/rzrqjygk%s.xls' % date2
        response = self.do_request(url, None, method='GET', type='binary')
        if response is not None:
            excel = pd.ExcelFile(io.BytesIO(response))
            df_total = excel.parse('汇总信息').dropna()
            df_detail = excel.parse('明细信息').dropna()
            df_total['date'] = date
            df_detail['date'] = date
            return df_total, df_detail
        else:
            return None, None

    def get_pledge_info(self, date):
        date2 = date_convert(date, '%Y-%m-%d', '%Y%m%d')
        url = 'http://query.sse.com.cn/exportExcel/exportStockPledgeExcle.do?tradeDate=%s' % (date2)
        response = self.do_request(url, None, method='GET', type='binary')
        if response is not None:
            excel = pd.ExcelFile(io.BytesIO(response))
            df_total = excel.parse('交易金额汇总').dropna()
            df_detail = excel.parse('交易数量明细').dropna()
            df_total['date'] = date
            df_detail['date'] = date
            return df_total, df_detail
        else:
            return None, None


class SZExAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    def get_index_list(self):
        # url = 'http://www.szse.cn/szseWeb/ShowReport.szse'
        url = 'http://www.szse.cn/api/report/ShowReport'
        data = {
            'SHOWTYPE': 'xls',
            'CATALOGID': '1812',
        }

        response = self.do_request(url, data, method='GET', type='binary')
        df = pd.read_excel(io.BytesIO(response))
        return df

    def get_index_component(self, index):
        # url = 'http://www.szse.cn/szseWeb/ShowReport.szse'
        url = 'http://www.szse.cn/api/report/ShowReport'
        data = {
            'SHOWTYPE': 'xls',
            'CATALOGID': '1747',
            'ZSDM': index
        }

        response = self.do_request(url, data, method='GET', type='binary')
        if response is not None:
            df = pd.read_excel(io.BytesIO(response))
            return df
        else:
            return None

    def get_rzrq_info(self, date):
        """
        融资融券 深交所
        http://www.szse.cn/disclosure/margin/margin/index.html
        :param date:
        :return:
        """
        df_total = self._get_rzrq_total(date)
        df_detail = self._get_rzrq_detail(date)
        if df_total is not None:
            df_total['date'] = date
        if df_detail is not None:
            df_detail['date'] = date
        return df_total, df_detail

    def _get_rzrq_total(self, date):
        """
        http://www.szse.cn/api/report/ShowReport?SHOWTYPE=xlsx&CATALOGID=1837_xxpl&txtDate=2020-12-24&tab2PAGENO=1&random=0.6629980185274793&TABKEY=tab2
        :param date:
        :return:
        """
        url = 'http://www.szse.cn/api/report/ShowReport'
        data = {
            'SHOWTYPE': 'xls',
            'CATALOGID': '1837_xxpl',
            'TABKEY': 'tab1',
            "txtDate": date,
        }

        response = self.do_request(url, data, method='GET', type='binary')
        if response is not None and len(response) > 0:
            df = pd.read_excel(io.BytesIO(response))
            return df
        else:
            return None

    def _get_rzrq_detail(self, date):
        # url = 'http://www.szse.cn/szseWeb/ShowReport.szse'
        url = 'http://www.szse.cn/api/report/ShowReport'
        data = {
            'SHOWTYPE': 'xls',
            'CATALOGID': '1837_xxpl',
            'TABKEY': 'tab2',
            "txtDate": date,
        }

        response = self.do_request(url, data, method='GET', type='binary')
        if response is not None and len(response) > 0:
            df = pd.read_excel(io.BytesIO(response))
            return df
        else:
            return None

    def get_pledge_info(self, date):
        df_total = self._get_pledge_info_total(date)
        df_detail = self._get_pledge_info_detail(date)
        if df_total is not None:
            df_total['date'] = date
        if df_detail is not None:
            df_detail['date'] = date
            df_detail['证券代码'] = df_detail['证券代码'].apply(lambda x: str(x).zfill(6))
        return df_total, df_detail

    def _get_pledge_info_total(self, date):
        # url = 'http://www.szse.cn/szseWeb/ShowReport.szse'
        url = 'http://www.szse.cn/api/report/ShowReport'
        data = {
            'SHOWTYPE': 'xls',
            'CATALOGID': '1837_gpzyhgxx',
            'TABKEY': 'tab1',
            "txtDate": date,
            'ENCODE': 1,
        }

        response = self.do_request(url, data, method='GET', type='binary')
        if response is not None and len(response) > 0:
            df = pd.read_excel(io.BytesIO(response))
            return df
        else:
            return None

    def _get_pledge_info_detail(self, date):
        # url = 'http://www.szse.cn/szseWeb/ShowReport.szse'
        url = 'http://www.szse.cn/api/report/ShowReport'
        data = {
            'SHOWTYPE': 'xls',
            'CATALOGID': '1837_gpzyhgxx',
            'TABKEY': 'tab2',
            "txtDate": date,
            'ENCODE': 1,
        }

        response = self.do_request(url, data, method='GET', type='binary')
        if response is not None and len(response) > 0:
            df = pd.read_excel(io.BytesIO(response))
            return df
        else:
            return None


class CSIAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    def get_index_list(self):
        url = 'http://www.csindex.com.cn/zh-CN/indices/index'

        page = 1
        result_data = []
        while True:
            data = {
                "data_type": "json",
                "page": page,
            }
            response = self.do_request(url, data, method='GET')
            rsp = json.loads(response)

            page = page + 1
            print("fetching data at page %d" % (page))
            if "list" in rsp:
                result_data.extend(rsp['list'])
                if len(rsp['list']) == 0:
                    break
            else:
                return None

        return pd.DataFrame(result_data)

    def get_index_component(self, index):
        url = 'http://www.csindex.com.cn/uploads/file/autofile/cons/%scons.xls' % (index)

        response = self.do_request(url, None, method='GET', type='binary')
        if response is not None:
            df = pd.read_excel(io.BytesIO(response))
            return df
        else:
            return None


class XueqiuAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    # 600000.SH -> SH600000
    def convert_to_xq_symbol(self, symbol):
        temp = symbol.split(".")
        return temp[1] + temp[0]

    def convert_to_xq_symbols(self, symbols):
        result = ''
        for symbol in symbols.split(','):
            result = result + self.convert_to_xq_symbol(symbol) + ','
        return result

    # SH600000 -> 600000.SH
    def convert_from_xq_symbol(self, symbol):
        market = symbol[0:2]
        code = symbol[2:]
        return code + '.' + market

    def prepare_cookies(self, url):
        response = self.do_request(url, None)
        if response is not None:
            cookies = self.get_cookies()
            return cookies
        else:
            return None

    def get_quote(self, symbols):
        url = 'https://stock.xueqiu.com/v5/stock/realtime/quotec.json'
        data = {
            'symbol': self.convert_to_xq_symbols(symbols)
        }

        # {"data":[{"symbol":"SH000001","current":3073.8321,"percent":-1.15,"chg":-35.67,"timestamp":1528427643770,"volume":6670380300,"amount":8.03515860132E10,"market_capital":1.393367880255658E13,"float_market_capital":1.254120000811718E13,"turnover_rate":0.64,"amplitude":0.91,"high":3100.6848,"low":3072.5418,"avg_price":3073.832,"trade_volume":5190400,"side":0,"is_trade":true,"level":1,"trade_session":null,"trade_type":null}],"error_code":0,"error_description":null}
        response = self.do_request(url, data, method='GET')

        if response is not None:
            jsonobj = json.loads(response)
            if jsonobj['error_code'] == 0:
                result = []
                for rsp in jsonobj['data']:
                    result.append({
                        'time': datetime.datetime.fromtimestamp(rsp['timestamp'] / 1000),
                        'symbol': self.convert_from_xq_symbol(rsp['symbol']),
                        'high': rsp['high'],
                        'low': rsp['low'],
                        'last': rsp['current'],
                        'change': rsp['chg'],
                        'percent': rsp['percent'],
                        'volume': rsp['volume'],
                        'amount': rsp['amount'],
                        'turnover_rate': rsp['turnover_rate'],
                        'market_capital': rsp['market_capital'],
                        'float_market_capital': rsp['float_market_capital'],
                        'is_trading': rsp['is_trade'],
                    })

                return pd.DataFrame(result), ''
            else:
                return None, jsonobj['error_description']
        else:
            return None, '请求数据失败'

    def get_kline(self, symbol, timestamp, period, count):
        url = 'https://stock.xueqiu.com/v5/stock/chart/kline.json'
        data = {
            'symbol': self.convert_to_xq_symbol(symbol),
            'begin': timestamp,
            'period': period,
            'type': 'before',
            'count': count,
            'indicator': 'kline',
        }

        cookies = self.prepare_cookies('https://xueqiu.com/hq')

        response = self.do_request(url, data, cookies=cookies, method='GET')
        if response is not None:
            jsonobj = json.loads(response)
            if jsonobj['error_code'] == 0:
                result = []
                for rsp in jsonobj['data']['item']:
                    result.append({
                        'symbol': symbol,
                        'time': datetime.datetime.fromtimestamp(rsp[0] / 1000),
                        'volume': rsp[1],
                        'open': rsp[2],
                        'high': rsp[3],
                        'low': rsp[4],
                        'last': rsp[5],
                        'change': rsp[6],
                        'percent': rsp[7],
                        'turnover_rate': rsp[8],
                    })

                return pd.DataFrame(result), ''
            else:
                return None, jsonobj['error_description']
        else:
            return None, '请求数据失败'

    def get_kline_multisymbol(self, symbols, timestamp, period, count):

        cookies = self.prepare_cookies('https://xueqiu.com/hq')
        url = 'https://stock.xueqiu.com/v5/stock/chart/kline.json'

        result = []
        for symbol in symbols:

            data = {
                'symbol': self.convert_to_xq_symbol(symbol),
                'begin': timestamp,
                'period': period,
                'type': 'before',
                'count': count,
                'indicator': 'kline',
            }

            response = self.do_request(url, data, cookies=cookies, method='GET')

            if response is not None:
                jsonobj = json.loads(response)
                if jsonobj['error_code'] == 0:
                    for rsp in jsonobj['data']['item']:
                        result.append({
                            'symbol': symbol,
                            'time': datetime.datetime.fromtimestamp(rsp[0] / 1000),
                            'volume': rsp[1],
                            'open': rsp[2],
                            'high': rsp[3],
                            'low': rsp[4],
                            'last': rsp[5],
                            'change': rsp[6],
                            'percent': rsp[7],
                            'turnover_rate': rsp[8],
                        })

        return pd.DataFrame(result), ''

    def get_kline_multitimestamp(self, symbol, timestamps, period, count):

        cookies = self.prepare_cookies('https://xueqiu.com/hq')
        url = 'https://stock.xueqiu.com/v5/stock/chart/kline.json'

        result = []
        for timestamp in timestamps:
            data = {
                'symbol': self.convert_to_xq_symbol(symbol),
                'begin': timestamp,
                'period': period,
                'type': 'before',
                'count': count,
                'indicator': 'kline',
            }

            response = self.do_request(url, data, cookies=cookies, method='GET')

            if response is not None:
                jsonobj = json.loads(response)
                if jsonobj['error_code'] == 0:
                    for rsp in jsonobj['data']['item']:
                        result.append({
                            'symbol': symbol,
                            'time': datetime.datetime.fromtimestamp(rsp[0] / 1000),
                            'volume': rsp[1],
                            'open': rsp[2],
                            'high': rsp[3],
                            'low': rsp[4],
                            'last': rsp[5],
                            'change': rsp[6],
                            'percent': rsp[7],
                            'turnover_rate': rsp[8],
                        })

        return pd.DataFrame(result), ''


class SinaAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    @staticmethod
    def clear_text(text):
        return text.replace('\n', '').strip()

    def get_adj_factor(self, symbol):
        now = datetime.datetime.now()
        year = now.year
        month = now.month
        if month < 4:
            quarter = 1
        elif month < 7:
            quarter = 2
        elif month < 10:
            quarter = 3
        else:
            quarter = 4

        temp = symbol.split(".")
        url = 'http://vip.stock.finance.sina.com.cn/corp/go.php/vMS_FuQuanMarketHistory/stockid/%s.phtml' % temp[0]

        curr_year = year
        curr_quarter = quarter
        result_list = []
        no_data_cnt = 0
        while True:
            print('getting data for year = %d, quarter = %d' % (curr_year, curr_quarter))
            param = {
                'year': curr_year,
                'jidu': curr_quarter,
            }
            response = self.do_request(url, param, method='GET', encoding='gb18030')
            soup = BeautifulSoup(response, "html5lib")
            divs = soup.find_all('div')

            data = []
            for div in divs:
                if div.has_attr('class') and 'tagmain' in div['class']:
                    tables = div.find_all('table')

                    for table in tables:
                        if table.has_attr('id') and table['id'] == 'FundHoldSharesTable':
                            rows = table.findAll('tr')
                            for row in rows:
                                cols = row.findAll('td')
                                if len(cols) == 8:
                                    date = SinaAgent.clear_text(cols[0].text)
                                    adjust_factor = SinaAgent.clear_text(cols[7].text)

                                    if date == '日期':
                                        continue

                                    data.append({
                                        "date": date,
                                        "adjust_factor": adjust_factor,
                                    })

            result_list.extend(data)
            if len(data) == 0:
                no_data_cnt = no_data_cnt + 1
                if no_data_cnt >= 3:
                    break

            # prepare for next round
            if curr_quarter == 1:
                curr_year = curr_year - 1
                curr_quarter = 4
            else:
                curr_quarter = curr_quarter - 1

        return pd.DataFrame(result_list), ""

    # 600000.SH -> SH600000
    def convert_to_sina_symbol(self, symbol):
        temp = symbol.split(".")
        return temp[1].lower() + temp[0]

    def get_trade_detail(self, symbol, trade_date):
        assert '服务已下线'


class CNInfoAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    @staticmethod
    def clear_text(text):
        return text.replace('\n', '').strip()

    def _parse_report_file(self, file):
        lines = file.readlines()
        data_list = []
        for i in range(len(lines)):
            items = lines[i].decode('gbk').split()
            if items[0][:2] == '机构':
                head = items[0].split(sep=',')
            else:
                items = lines[i].decode('gbk')[1:]
                data = items.split(sep=',')
                data[0] = data[0][1:-1]
                data[-1] = remove_non_numerical(data[-1])
                data_list.append(data)
        df = pd.DataFrame(data_list)
        df.columns = head
        return df

    def get_report_data(self, market, symbol, type):

        url = 'http://www.cninfo.com.cn/cninfo-new/data/download'
        data = {
            'market': market,
            'type': type,
            'code': symbol,
            'orgid': 'gs%s%s' % (market, symbol),
            'minYear': '1990',
            'maxYear': '2018',
        }

        response = self.do_request(url, param=data, method='POST', type='binary')
        '''if response is None:
            return None, '没有获取到数据'
        else:
        '''
        try:
            zip_ref = zipfile.ZipFile(io.BytesIO(response))
            df_list = []
            for finfo in zip_ref.infolist():
                file = zip_ref.open(finfo, 'r')
                df = self._parse_report_file(file)
                df_list.append(df)
            df_result = _concat_df(df_list)
            df_result.reset_index(inplace=True, drop=True)

            return df_result, ''
        except:
            return None, '获取数据失败'

    def get_shareholder_structure(self, market, symbol):

        if symbol.startswith('002'):
            board = 'sme'
        elif symbol.startswith('3'):
            board = 'cn'
        else:
            board = 'mb'

        url = 'http://www.cninfo.com.cn/information/lastest/%s%s%s.html' % (market, board, symbol)
        response = self.do_request(url, encoding='gb18030')
        if response is None:
            return None, '获取数据失败'

        soup = BeautifulSoup(response, "html5lib")
        divs = soup.find_all('div')

        data = []
        for div in divs:
            if div.has_attr('class') and 'clear' in div['class']:
                tables = div.find_all('table')

                for table in tables:
                    rows = table.findAll('tr')
                    for row in rows:
                        cols = row.findAll('td')
                        if len(cols) == 2:
                            indicator = CNInfoAgent.clear_text(cols[0].text).replace('：', '')
                            value = CNInfoAgent.clear_text(cols[1].text)

                            data.append({
                                "indicator": indicator,
                                "value": value,
                            })
                break

        return pd.DataFrame(data), ""

    def get_dividend(self, symbol):
        symbol = symbol[:6]
        url = "http://www.cninfo.com.cn/information/dividend/szmb%s.html"
        response = self.do_request(url % symbol, method='GET', encoding='gbk')
        if response is None:
            return pd.DataFrame([])
        soup = BeautifulSoup(response, 'html5lib')
        # get name_cn
        tds = soup.find_all('td')
        for td in tds:
            if td.has_attr('style') and 'padding-right:10px' in td['style']:
                name_cn = td.text.split('：')[-1]

        # get dividend_data
        divs = soup.find_all('div')
        for div in divs:
            if div.has_attr('class') and 'clear' in div['class']:
                trs = div.find_all('tr')
                if trs == []:
                    continue
                data_list = []
                for tr in trs[1:]:
                    data = [symbol, name_cn]
                    tds = tr.find_all('td')
                    for td in tds:
                        text = td.text.replace(' ', '').replace('\n', '').replace('\xa0', '')
                        data.append(text)
                    data_list.append(data)
                df_res = pd.DataFrame(data_list, columns=['股票代码', '公司名称', '分红年度', '分红方案', '股权登记日',
                                                          '除权日', '红股上市日'])
                df_res['股权登记日'] = df_res['股权登记日'].map(time_map)
                df_res['除权日'] = df_res['除权日'].map(time_map)
                df_res['分红方案'] = df_res['分红方案'].map(plan_map)
                df_res['税后股利'] = df_res['分红方案'].map(lambda x: 0.8 * float(x))
                df_res['公司代码'] = df_res['股票代码']
                df = df_res[['公司代码', '股权登记日', '分红方案', '税后股利', '除权日', '公司名称', '股票代码']]
                df.columns = ['COMPANY_CODE', 'DIVIDEND_DATE', 'DIVIDEND_PER_SHARE1_A',
                              'DIVIDEND_PER_SHARE2_A', 'EX_DIVIDEND_DATE_A', 'SECURITY_ABBR_A', 'SECURITY_CODE_A']
                return df


class EastMoneyAgent(RestAgent):
    """
    大盘资金流向：http://data.eastmoney.com/zjlx/dpzjlx.html

    """

    def __init__(self):
        RestAgent.__init__(self)

    def _parse_hist_money_flow(self, response):
        jsonobj = json.loads(response)
        result = []
        columns_list = ['Time', 'ZLJLRJE', 'XDJLRJE', 'ZDJLRJE', 'DDJLRJE',
                        'CDDJLRJE', 'ZLJLRZB', 'XDJLRZB', 'ZDJLRZB', 'DDJLRZB',
                        'CDDJLRZB', 'SZSPJ', 'SZZDF', 'SZSPJ', 'SZZDF']
        for data in jsonobj['data']['klines']:
            items = data.split(',')
            result.append(dict(zip(columns_list, items)))
        return pd.DataFrame(result)

    def _get_hist_money_flow(self, url):
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'

        df = self._parse_hist_money_flow(response)
        return df, ''

    def get_hist_money_flow(self, symbol):
        url = 'http://ff.eastmoney.com//EM_CapitalFlowInterface/api/js?type=hff&rtntype=2&js={"data":(x)}&check=TMLBMSPROCR&acces_token=1942f5da9b46b069953c873404aad4b5&id=%s' % symbol
        return self._get_hist_money_flow(url)

    def get_hist_money_flow_market(self):
        """
        大盘资金流向： http://data.eastmoney.com/zjlx/dpzjlx.html
        接口： http://push2his.eastmoney.com/api/qt/stock/fflow/daykline/get?lmt=0&klt=101&secid=1.000001&secid2=0.399001&fields1=f1,f2,f3,f7&fields2=f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f62,f63,f64,f65&ut=b2884a393a59ad64002292a3e90d46a5&cb=jQuery18308780832859698127_1608864632777
        :return:
        """
        url = 'http://push2his.eastmoney.com/api/qt/stock/fflow/daykline/get?lmt=0' \
              '&klt=101&secid=1.000001&secid2=0.399001&fields1=f1,f2,f3,f7' \
              '&fields2=f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f62,f63,f64,f65' \
              '&ut=b2884a393a59ad64002292a3e90d46a5&cb=jQuery18308780832859698127_1608864632777'
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'

        # get data from html
        pattern = re.compile(r'\d+\_\d+\((.*?)\)')
        json_rsp = pattern.findall(response)[0]
        df = self._parse_hist_money_flow(json_rsp)
        return df, ''

    def _get_realtime_money_flow(self, url):
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'

        jsonobj = json.loads(response)
        result = {}
        result['Time'] = jsonobj['xa'].split(',')
        result['ZLJLRJE'] = list()
        result['CDDJLRJE'] = list()
        result['DDJLRJE'] = list()
        result['ZDJLRJE'] = list()
        result['XDJLRJE'] = list()
        for data in jsonobj['ya']:
            items = data.split(',')
            result['ZLJLRJE'].append(items[0])
            result['CDDJLRJE'].append(items[1])
            result['DDJLRJE'].append(items[2])
            result['ZDJLRJE'].append(items[3])
            result['XDJLRJE'].append(items[4])

        df = pd.DataFrame().from_dict(result, orient='index').T
        df.dropna(inplace=True)
        return df, ''

    def get_realtime_money_flow(self, symbol):
        url = 'http://ff.eastmoney.com/EM_CapitalFlowInterface/api/js?id=%s&type=ff&check=MLBMS&js={(x)}&rtntype=3&acces_token=1942f5da9b46b069953c873404aad4b5' % symbol
        return self._get_realtime_money_flow(url)

    def get_realtime_money_flow_market(self):
        url = 'http://ff.eastmoney.com/EM_CapitalFlowInterface/api/js?id=ls&type=ff&check=MLBMS&js={(x)}&rtntype=3&acces_token=1942f5da9b46b069953c873404aad4b5'
        return self._get_realtime_money_flow(url)

    # ==============================================================================
    # 一次性获取所有股票的实时资金流，并按照主力净流入净额排序
    ## 指标定义
    # 超大单：大于等于50万股或者100万元的成交单;
    # 大单：大于等于10万股或者20万元且小于50万股和100万元的成交单;
    # 中单：大于等于2万股或者4万元且小于10万股和20万元的成交单;
    # 小单：小于2万股和4万元的成交单;
    # 流入：买入成交额;
    # 流出：卖出成交额;
    # 主力流入：超大单加大单买入成交额之和;
    # 主力流出：超大单加大单卖出成交额之和;
    # 净额：流入-流出;
    # 净比：(流入-流出)/总成交额;
    # 单位：亿元
    # ==============================================================================
    def toDataFrame(self, ll):
        dataframe = []
        for l in ll:
            l = l.replace('-', '0')
            temp = l.split(",")[1:]
            temp[2:-2] = map(eval, temp[2:-2])
            dataframe.append(temp)
        dataframe = pd.DataFrame(dataframe)
        dataframe.columns = [u'代码', u'名称', u'最新价', u'今日涨跌幅', u'今日主力净流入净额', u'今日主力净流入净占比', u'今日超大单净流入净额', u'今日超大单净流入净占比',
                             u'今日大单净流入净额', u'今日大单净流入净占比', u'今日中单净流入净额', u'今日中单净流入净占比', u'今日小单净流入净额', u'今日小单净流入净占比',
                             u'time', u'未知']
        return dataframe

    def _get_realtime_allstock_flow(self, url):
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'
        pages = 'pages'
        date = 'date'
        data = 'data'
        data = eval(response[13:])
        flashflow = data['data']
        df = self.toDataFrame(flashflow)
        df.index = df.ix[:, 0]
        df.dropna(inplace=True)
        return df, ''

    def get_allstock_flow(self):
        url = 'http://nufm.dfcfw.com/EM_Finance2014NumericApplication/JS.aspx?type=ct&st=(BalFlowMain)&sr=-1&p=1&ps=3700&js=var%20ucjEIgIa={pages:(pc),date:%222014-10-22%22,data:[(x)]}&token=1942f5da9b46b069953c873404aad4b5&cmd=C._AB&sty=DCFFITA&rt=50984894'
        return self._get_realtime_allstock_flow(url)



# encoding: UTF-8

import datetime
from .aqi_agent import AQIAgent

aqi_agent = AQIAgent()

def set_proxies(proxies):
    aqi_agent.set_proxies(proxies)

def get_hour_aqi(time = None):
    if time is None:
        i = 0
        while i < 3:
            pub_time = datetime.datetime.now() - datetime.timedelta(hours=i)
            time = datetime.datetime.strftime(pub_time, "%Y-%m-%d %H:00:00")
            df_aqi = aqi_agent.get_hour_aqi(time)
            if len(df_aqi) > 0:
                return df_aqi
            i = i + 1
        return None

    else:
        return aqi_agent.get_hour_aqi(time)

def get_hour_aqi_onecity(city, date = None):
    if date is None:
        date = datetime.datetime.strftime(datetime.datetime.now(), "%Y-%m-%d")
    return aqi_agent.get_hour_aqi_onecity(city, date)

# can only get history data
def get_daily_aqi(date):
    return aqi_agent.get_daily_aqi(date)

# can only get history data
def get_daily_aqi_onecity(city):
    return aqi_agent.get_daily_aqi_onecity(city)

# recent 30 days
def get_recent_daily_aqi_onecity(city):
    return aqi_agent.get_recent_daily_aqi_onecity(city)


from bs4 import BeautifulSoup
import pandas as pd
import numpy as np
import datetime

from opendatatools.common import RestAgent, get_current_day, date_convert

class HKExAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    @staticmethod
    def clear_text(text):
        return text.replace('\n', '').strip()

    # hongkong exchange 90000 -> 600000
    @staticmethod
    def process_code(market, text):
        code = HKExAgent.clear_text(text)
        if market == "SH":
            return "60" + code[1:] + ".SH"
        else:
            if code.startswith('77'):
                return "300" + code[2:] + ".SZ"
            else:
                return "00" + code[1:] + ".SZ"

    def get_lgt_share(self, market='SH', date = None):
        url = "http://sc.hkexnews.hk/TuniS/www.hkexnews.hk/sdw/search/mutualmarket_c.aspx?t=%s" % (market)

        today = get_current_day()
        if date is None:
            date = today

        data = {
            'today' : date_convert(today, "%Y-%m-%d", "%Y%m%d"),
            'sortBy': 'stockcode',
            'sortDirection': 'asc',
            'alertMsg': '',
            'txtShareholdingDate': date_convert(date, "%Y-%m-%d", "%Y/%m/%d"),
            'btnSearch' : "搜寻",
        }
        aspx_param = self.get_aspx_param(url)
        data.update(aspx_param)

        self.add_headers({
            "Content-Type" : "application/x-www-form-urlencoded",
        })

        rsp = self.do_request(url, data, "POST")

        # 2. 开始解析返回数据，并从html中提取需要的内容
        data = list()
        soup = BeautifulSoup(rsp, "html5lib")
        divs = soup.find_all('div')

        result_date = ""
        for div in divs:
            if div.has_attr('id') and 'pnlResult' in div['id']:

                res_span = div.find_all('span')

                if (len(res_span)) > 0:
                    result_date = HKExAgent.clear_text(res_span[0].text)
                    result_date = result_date.replace("持股日期:", "").strip()
                    result_date = date_convert(result_date, "%Y/%m/%d", "%Y-%m-%d")
                rows = div.table.findAll('tr')
                for row in rows:
                    cols = row.findAll('td')
                    if len(cols) == 4:
                        code = HKExAgent.clear_text(cols[0].text.replace("股份代号:", ""))
                        name = HKExAgent.clear_text(cols[1].text.replace("股份名称:", ""))
                        share_num  = HKExAgent.clear_text(cols[2].text.replace("于中央结算系统的持股量:", ""))
                        percent = HKExAgent.clear_text(cols[3].text.replace("占于深交所上市及交易的A股总数的百分比:", "").replace("占于上交所上市及交易的A股总数的百分比:", ""))
                        data.append({
                            "code": HKExAgent.process_code(market, code),
                            "name": name,
                            "share_num": share_num,
                            "percent": percent,
                            "date" : result_date,
                        })

        df = pd.DataFrame(data)
        return df


# encoding: utf-8

from .fund_agent import EastMoneyAgent

eastmoney_agent = EastMoneyAgent()

def get_fund_list():
    return eastmoney_agent.get_fund_list()

def get_fund_company():
    return eastmoney_agent.get_fund_company()

def get_fundlist_by_company(companyid):
    return eastmoney_agent.get_fundlist_by_company(companyid)

def get_fund_type():
    return eastmoney_agent.get_fund_type()

def get_fundlist_by_type(type):
    return eastmoney_agent.get_fundlist_by_type(type)

def get_fund_nav(fund_code):
    return eastmoney_agent.get_fund_nav(fund_code)


# encoding: utf-8
from opendatatools import aqi
import unittest

class TestAQI(unittest.TestCase):

    # 获取某日全国各大城市的AQI数据
    def test_get_daily_aqi(self):
        df = aqi.get_daily_aqi('2018-01-01')
        assert(len(df)>0)

    # 获取单个城市的AQI历史数据
    def test_get_daily_aqi_onecity(self):
        df = aqi.get_daily_aqi_onecity('北京市')
        assert(len(df)>0)

    # 获取单个城市某日的AQI小时数据
    def test_get_hour_aqi_onecity(self):
        aqi_hour = aqi.get_hour_aqi_onecity('北京市', '2018-05-26')
        assert(len(aqi_hour)>0)

    #获取实时AQI小时数据
    def test_get_hour_aqi(self):
        aqi_hour = aqi.get_hour_aqi()
        assert (len(aqi_hour) > 0)




# encoding: utf-8

import datetime
from .nbs_agent import NBSAgent

nbs_agent = NBSAgent()

def convert_date(date, date_format):
    if date_format == '%YQ':
        date_format = '%Y%m'
        date = date.replace('A', '03').replace('B', '06').replace('C', '09').replace('D', '12')

    return datetime.datetime.strptime(date, date_format)

def disp_result(df, date_format = '%Y%m'):
    import matplotlib.pyplot as plt
    import datetime

    from matplotlib.pylab import mpl
    # 指定默认字体
    mpl.rcParams['font.sans-serif'] = ['FangSong']
    # 解决保存图像是负号'-'显示为方块的问题
    mpl.rcParams['axes.unicode_minus'] = False

    fig = plt.figure(figsize=(16,10))
    for name, sub_df in df.groupby('indicator_name'):
        sub_df['date'] = sub_df['date'].apply(lambda x : convert_date(x, date_format))
        sub_df.set_index('date', inplace = True)
        sub_df.sort_index(ascending=True, inplace=True)
        plt.plot(sub_df.index, sub_df['value'], label=name)
        plt.legend()

    plt.grid(True)
    plt.show()

# 指标名称映射表
def get_indicator_map():
    return nbs_agent.get_indicator_map()

# 省份名称
def get_region_map():
    return nbs_agent.get_region_map()

# 城市名称
def get_city_map():
    return nbs_agent.get_city_map()

# 年度数据
def get_gdp_y():
    return nbs_agent.get_gdp_y()

def get_region_gdp_y():
    return nbs_agent.get_region_gdp_y()

def get_population_size_y():
    return nbs_agent.get_population_size_y()

def get_population_structure_y():
    return nbs_agent.get_population_structure_y()

def get_house_price_index(region):
    return nbs_agent.get_house_price_index(region)

def get_cpi():
    return nbs_agent.get_cpi()

def get_region_cpi(region):
    return nbs_agent.get_region_cpi(region)

def get_ppi():
    return nbs_agent.get_ppi()

def get_region_ppi(region):
    return nbs_agent.get_region_ppi(region)

def get_gdp():
    return nbs_agent.get_gdp()

def get_region_gdp(region):
    return nbs_agent.get_region_gdp(region)

def get_gdp_q2q():
    return nbs_agent.get_gdp_q2q()

def get_M0_M1_M2():
    return nbs_agent.get_M0_M1_M2()

def get_fiscal_revenue():
    return nbs_agent.get_fiscal_revenue()

def get_fiscal_expend():
    return nbs_agent.get_fiscal_expend()

def get_manufacturing_pmi():
    return nbs_agent.get_manufacturing_pmi()

def get_non_manufacturing_pmi():
    return nbs_agent.get_non_manufacturing_pmi()

def get_pmi():
    return nbs_agent.get_pmi()

def get_import_export():
    return nbs_agent.get_import_export()

def get_fdi():
    return nbs_agent.get_fdi()

def get_retail_sales():
    return nbs_agent.get_retail_sales()

def get_online_retail_sales():
    return nbs_agent.get_online_retail_sales()

def get_realestate_investment():
    return nbs_agent.get_realestate_investment()

def get_region_realestate_investment(region):
    return nbs_agent.get_region_realestate_investment(region)

def get_fixed_asset_investment():
    return nbs_agent.get_fixed_asset_investment()

def get_region_fixed_asset_investment(region):
    return nbs_agent.get_region_fixed_asset_investment(region)


# encoding: UTF-8

def remove_chinese(str):
    s = ""
    for w in str:
        if w >= u'\u4e00' and w <= u'\u9fa5':
            continue
        s += w
    return s


def remove_non_numerical(s):
    f = ''
    for i in range(len(s)):
        try:
            f = float(s[:i + 1])
        except:
            return f
    return str(f)



# encoding: utf-8

from opendatatools.common import RestAgent
import io
import pandas as pd


class WorldCupAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

        self._data_df = None

    def load_data(self):
        url = 'https://github.com/PKUJohnson/OpenData/raw/master/3rd/WorldCupMatches.xls'
        response = self.do_request(url, None, method='GET', type='binary')
        if response is not None:
            excel = pd.ExcelFile(io.BytesIO(response))
            self._data_df = excel.parse('WorldCupMatches').dropna()
            return self._data_df

        return None

    def _get_winner(self, row):
        team1 = row['Home Team Name']
        score1 = row['Home Team Goals']
        team2 = row['Away Team Name']
        score2 = row['Away Team Goals']
        win_cond = row['Win conditions']

        if score1 > score2:
            winner = team1
        elif score1 < score2:
            winner = team2
        else:
            winner = win_cond.split(" ")[0]

        return winner

    def _get_goal_stat(self, df):
        goal  = 0
        games = 0
        for index, row in df.iterrows():
            score1 = row['Home Team Goals']
            score2 = row['Away Team Goals']

            goal  = goal + score2 + score1
            games = games + 1

        return goal, games, goal/games

    def _get_champion_team(self):
        df_final = self._data_df[self._data_df.Stage == 'Final']
        dict_champion = {}
        for index, row in df_final.iterrows():
            year     = row['Year']
            champion = self._get_winner(row)
            dict_champion[year] = champion

        # 1950 年没有决赛
        dict_champion[1950] = 'Uruguay'
        return dict_champion

    def get_champion_rank(self):
        _get_champion_team = self._get_champion_team()
        dict_chanpion = {}
        for year, champion in _get_champion_team.items():
            if champion in dict_chanpion:
                dict_chanpion[champion] = dict_chanpion[champion] + 1
            else:
                dict_chanpion[champion] = 1

        df = pd.DataFrame(dict_chanpion, index = [0]).T
        df.columns = ['number of champions']
        df.sort_values('number of champions', ascending=False, inplace=True)
        return df

    def get_finalgame_rank(self):
        df_final = self._data_df[self._data_df.Stage == 'Final']
        dict_game = {}
        for index, row in df_final.iterrows():
            team1 = row['Home Team Name']
            team2 = row['Away Team Name']

            if team1 in dict_game:
                dict_game[team1] = dict_game[team1] + 1
            else:
                dict_game[team1] = 1

            if team2 in dict_game:
                dict_game[team2] = dict_game[team2] + 1
            else:
                dict_game[team2] = 1

        df = pd.DataFrame(dict_game, index=[0]).T
        df.columns = ['number of final games']
        df.sort_values('number of final games', ascending=False, inplace=True)
        return df

    def get_wingame_rank(self):
        dict_winner = {}
        for index, row in self._data_df.iterrows():
            winner = self._get_winner(row)
            if winner.strip() == '':
                continue

            if winner in dict_winner:
                dict_winner[winner] = dict_winner[winner] + 1
            else:
                dict_winner[winner] = 1

        df = pd.DataFrame(dict_winner, index = [0]).T
        df.columns = ['number of win games']
        df.sort_values('number of win games', ascending=False, inplace=True)
        return df

    def get_game_rank(self):
        dict_game = {}
        for index, row in self._data_df.iterrows():
            team1 = row['Home Team Name']
            team2 = row['Away Team Name']

            if team1 in dict_game:
                dict_game[team1] = dict_game[team1] + 1
            else:
                dict_game[team1] = 1

            if team2 in dict_game:
                dict_game[team2] = dict_game[team2] + 1
            else:
                dict_game[team2] = 1

        df = pd.DataFrame(dict_game, index=[0]).T
        df.columns = ['number of games']
        df.sort_values('number of games', ascending=False, inplace=True)
        return df

    def get_year_rank(self):
        dict_year = {}
        for index, row in self._data_df.iterrows():
            year  = row['Year']
            team1 = row['Home Team Name']
            team2 = row['Away Team Name']

            if team1 in dict_year:
                dict_year[team1].add(year)
            else:
                dict_year[team1] = {year}

            if team2 in dict_year:
                dict_year[team2].add(year)
            else:
                dict_year[team2] = {year}

        dict_year_num = {k :len(v) for k,v in dict_year.items()}

        df = pd.DataFrame(dict_year_num, index=[0]).T
        df.columns = ['number of year']
        df.sort_values('number of year', ascending=False, inplace=True)
        return df

    def _get_game_stat(self, df, team):
        dict_stat = {}
        for index, row in df.iterrows():
            team1 = row['Home Team Name']
            team2 = row['Away Team Name']
            if team1 == team or team2 == team:
                winner = self._get_winner(row)
                if winner == team:
                    stat = (1, 0, 0)
                elif winner == '':
                    stat = (0, 1, 0)
                else:
                    stat = (0, 0, 1)

                if team in dict_stat:
                    old_stat = dict_stat[team]
                    dict_stat[team] = (stat[0] + old_stat[0], stat[1] + old_stat[1], stat[2] + old_stat[2])

        return dict_stat

    def get_champion_stat(self):
        dict_champion_stat = []
        dict_champion = self._get_champion_team()
        for year, champion in dict_champion:
            df = self._data_df[self._data_df.Year == year]
            dict_stat = self._get_game_stat(df, champion)
            dict_champion_stat.append({
                'year' : year,
                'champion' : champion,
                'num of win'  : dict_stat[0],
                'num of tie'  : dict_stat[1],
                'num of loss' : dict_stat[2],
            })


        return dict_champion_stat

    def _get_firstgame_stat(self, df, team):

        for index, row in df.iterrows():
            year   = row['Year']
            team1  = row['Home Team Name']
            team2  = row['Away Team Name']
            score1 = row['Home Team Goals']
            score2 = row['Away Team Goals']
            win_cond = row['Win conditions']

            if team1 == team or team2 == team:
                winner = self._get_winner(row)
                if winner == team:
                    stat = '胜'
                elif winner == '':
                    stat = '平'
                else:
                    stat = '负'

                detail = {'year': year, 'team1' : team1, 'team2': team2, 'score': '%s : %s' % (score1, score2), 'result' : stat}
                return stat, detail

        return None, None


    def get_champion_fistgame_stat(self):
        dict_champion_fistgame_stat = {}
        list_champion_fistgame_detail = []
        dict_champion = self._get_champion_team()
        for year, champion in dict_champion.items():
            df = self._data_df[self._data_df.Year == year]
            stat, detail = self._get_firstgame_stat(df, champion)

            if stat in dict_champion_fistgame_stat:
                dict_champion_fistgame_stat[stat] = dict_champion_fistgame_stat[stat] + 1
            else:
                dict_champion_fistgame_stat[stat] = 1

            list_champion_fistgame_detail.append(detail)

        return pd.DataFrame([{'result' : k, 'num of games' : v} for k,v in dict_champion_fistgame_stat.items()]), pd.DataFrame(list_champion_fistgame_detail)

    def get_goal_stat(self):
        start_year = 1934
        end_year   = 2018
        list_goal_stat = []
        for year in range(start_year, end_year, 4):
            df = self._data_df[self._data_df.Year == year]
            if len(df) > 0:
                goal, games, avg_goal = self._get_goal_stat(df)
                list_goal_stat.append({'year': year, 'goal' : goal, 'games' : games, 'avg_goal': avg_goal})

        df = pd.DataFrame(list_goal_stat)
        df.sort_values('year', inplace=True)
        return df

    def _get_goal_stat_team(self, df, team):
        goal  = 0
        lose  = 0
        games = 0
        for index, row in df.iterrows():
            team1 = row['Home Team Name']
            team2 = row['Away Team Name']
            score1 = row['Home Team Goals']
            score2 = row['Away Team Goals']

            if team1 == team or team2 == team:
                if team1 == team:
                    goal = goal + score1
                    lose = lose + score2
                else:
                    goal = goal + score2
                    lose = lose + score1

                games = games + 1

        return goal, lose, games, goal/games, lose/games

    def get_champion_goal_stat(self):
        list_champion_goal_stat = []
        dict_champion = self._get_champion_team()
        for year, champion in dict_champion.items():
            df = self._data_df[self._data_df.Year == year]
            goal, lose, games, avg_goal, avg_lose = self._get_goal_stat_team(df, champion)

            list_champion_goal_stat.append({'year': year,
                                            'champion' : champion,
                                            'goal' : goal,
                                            'lose' : lose,
                                            'games': games,
                                            'avg_goal' : avg_goal,
                                            'avg_lose' : avg_lose,}
                                           )

        df = pd.DataFrame(list_champion_goal_stat)
        df.sort_values('year', inplace=True)

        return df


# encoding: utf-8
from opendatatools import hkex
import unittest

class TestHKEx(unittest.TestCase):

    # 获取陆港通 北向 持股 情况
    # 参数：市场SH/SZ，日期YYYY-MM-DD
    def test_get_lgt_share(self):
        df = hkex.get_lgt_share(market = 'SZ', date = '2018-05-28')
        assert(len(df) > 100)

        df = hkex.get_lgt_share(market='SH', date='2018-05-28')
        assert(len(df) > 100)

        df = hkex.get_lgt_share()
        assert(len(df) > 0)


# encoding: utf-8

from opendatatools.common import RestAgent
import json
import pandas as pd

class GaokaoAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    def _get_data(self, url):
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'

        jsonobj = json.loads(response)
        df = pd.DataFrame(jsonobj)
        return df, ''

    def get_school_list(self, text):
        url = 'https://tu.quantos.org/api/search/%s' % text
        return self._get_data(url)

    def get_school_baseinfo(self, school):
        url = 'https://tu.quantos.org/api/schoolinfo/baseinfo/%s' % school
        return self._get_data(url)

    def get_school_major(self, school):
        url = 'https://tu.quantos.org/api/schoolinfo/major/%s' % school
        return self._get_data(url)

    def get_school_score(self, school):
        url = 'https://tu.quantos.org/api/schoolinfo/score/%s/%%25/%%25' % school
        return self._get_data(url)

    def get_batch_score(self, province, subject):
        url = 'https://tu.quantos.org/api/batchscore/%s/%s/' % (province, subject)
        df, msg = self._get_data(url)
        return df, msg

if __name__ == "__main__":
    agent = GaokaoAgent()
    df, msg = agent.get_batch_score('北京', '理科')
    print(df)




import pandas as pd
import json
import datetime
from opendatatools.common import RestAgent

class CoinAgent(RestAgent):

    def __init__(self):
        RestAgent.__init__(self)
        self.set_url()

    def set_url(self):
        self.COIN_LIST_URL = 'https://www.cryptocompare.com/api/data/coinlist/'
        self.COIN_SNAPSHOT_FULL_BY_ID_URL = 'https://www.cryptocompare.com/api/data/coinsnapshotfullbyid/?id='
        self.COIN_SNAPSHOT_URL = 'https://www.cryptocompare.com/api/data/coinsnapshot/'

        self.PRICE_URL = 'https://min-api.cryptocompare.com/data/price'
        self.PRICE_MULTI_URL = 'https://min-api.cryptocompare.com/data/pricemulti'
        self.PRICE_MULTI_FULL_URL = 'https://min-api.cryptocompare.com/data/pricemultifull'
        self.PRICE_HISTORICAL_URL = 'https://min-api.cryptocompare.com/data/pricehistorical'

        self.GENERATE_AVG_URL = 'https://min-api.cryptocompare.com/data/generateAvg'
        self.DAY_AVG_URL = 'https://min-api.cryptocompare.com/data/dayAvg'

        self.SUBS_WATCH_LIST_URL = 'https://min-api.cryptocompare.com/data/subsWatchlist'
        self.SUBS_URL = 'https://min-api.cryptocompare.com/data/subs'

        self.ALL_EXCHANGES_URL = 'https://min-api.cryptocompare.com/data/all/exchanges'
        self.TOP_EXCHANGES_URL = 'https://min-api.cryptocompare.com/data/top/exchanges'
        self.TOP_VOLUMES_URL = 'https://min-api.cryptocompare.com/data/top/volumes'
        self.TOP_PAIRS_URL = 'https://min-api.cryptocompare.com/data/top/pairs'

        self.HIST_DAY_URL = 'https://min-api.cryptocompare.com/data/histoday'
        self.HIST_HOUR_URL = 'https://min-api.cryptocompare.com/data/histohour'
        self.HIST_MINUTE_URL = 'https://min-api.cryptocompare.com/data/histominute'

        self.SOCIAL_STATS_URL = 'https://www.cryptocompare.com/api/data/socialstats?id='

        self.MINING_CONTRACTS_URL = 'https://www.cryptocompare.com/api/data/miningcontracts/'
        self.MINING_EQUIPMENT_URL = 'https://www.cryptocompare.com/api/data/miningequipment/'

    def format_unix_time(self, df, col='time'):
        if col in df.columns:
            df[col] = df[col].apply(lambda x : datetime.datetime.fromtimestamp(x))

        return df

    def _fetch(self, url):
        rsp = self.do_request(url)
        json_obj = json.loads(rsp)

        if "Response" in json_obj:
            response = json_obj['Response']
            if "Message" in json_obj :
                message  = json_obj['Message']
            else:
                message = ""

            if response == "Success":
                data = json_obj['Data']
                return data, message
            else:
                return None, message
        else:
            return json_obj, ""

    def get_coin_list(self):
        data, message = self._fetch(self.COIN_LIST_URL)
        if data is None:
            return None, message

        df = self.format_unix_time(pd.DataFrame(data).T)
        return df, message

    def get_coin_snapshot(self, fsym, tsym):
        '''
            Keyword arguments:
            fsym - The symbol of the currency you want to get that for
            tsym - The symbol of the currency that data will be in.
        '''
        data, message = self._fetch(self.COIN_SNAPSHOT_URL + '?fsym=' + fsym + '&tsym=' + tsym)
        if data is None:
            return None, None, message

        df = pd.DataFrame(data['Exchanges'])
        df['LASTUPDATE'] = df['LASTUPDATE'].apply(lambda x : int(x))
        df = self.format_unix_time(df, 'LASTUPDATE')
        return data['AggregatedData'], df, message

    def get_coin_price(self, fsym, tsyms, exchange):
        data, message = self._fetch(self.PRICE_URL + '?fsym=' + fsym + '&tsyms=' + tsyms + "&e=" + exchange)
        if data is None:
            return None, message

        return data, message

    def get_exchange_list(self):
        data, message = self._fetch(self.ALL_EXCHANGES_URL)
        if data is None:
            return None, message

        return pd.DataFrame(data.keys()), message

    def get_his_min(self, fsym, tsym, exchange, limit):
        data, message = self._fetch(self.HIST_MINUTE_URL + '?fsym=' + fsym + '&tsym=' + tsym + "&e=" + exchange + "&limit=" + str(limit))
        if data is None:
            return None, message

        df = self.format_unix_time(pd.DataFrame(data))
        return df, message

    def get_his_hour(self, fsym, tsym, exchange, limit):
        data, message = self._fetch(self.HIST_HOUR_URL + '?fsym=' + fsym + '&tsym=' + tsym + "&e=" + exchange + "&limit=" + str(limit))
        if data is None:
            return None, message

        df = self.format_unix_time(pd.DataFrame(data))
        return df, message

    def get_his_day(self, fsym, tsym, exchange, limit):
        data, message = self._fetch(self.HIST_DAY_URL + '?fsym=' + fsym + '&tsym=' + tsym + "&e=" + exchange + "&limit=" + str(limit))
        if data is None:
            return None, message

        df = self.format_unix_time(pd.DataFrame(data))
        return df, message



# encoding: utf-8

from .anjuke_agent import AnjukeAgent, anjuke_city_map
from .lianjia_agent import LianjiaAgent
import pandas as pd

anjuke_agent  = AnjukeAgent()
lianjia_agent = LianjiaAgent()

def set_proxies(proxies):
    anjuke_agent.set_proxies(proxies)
    lianjia_agent.set_proxies(proxies)

def get_city_list():
    return anjuke_agent.get_city_list()

def get_real_house_price(city):
    return anjuke_agent.get_real_house_price(city)

def get_city_list_lianjia():
    return lianjia_agent.get_city_list()

def get_district_list_lianjia(city):
    return lianjia_agent.get_district_by_city(city)

def get_esf_list_lianjia(city, max_page_no=10):
    city_list = lianjia_agent.get_city_list()
    if city not in city_list:
        msg = "invalid city, please use " + ".".join(city_list)
        return None, msg

    return lianjia_agent.get_esf_list(city, max_page_no)

def get_esf_list_by_district_lianjia(city, district, max_page_no=10):
    city_list = lianjia_agent.get_city_list()
    if city not in city_list:
        msg = "invalid city, please use " + ".".join(city_list)
        return None, msg

    district_map = lianjia_agent.get_district_by_city(city)
    if district not in district_map:
        msg = "invalid district, please use " + ".".join(district_map.keys())
        return None, msg

    return lianjia_agent.get_esf_list_by_district(city, district, max_page_no)

def get_rent(city, page_no):
    if city in anjuke_city_map.keys():
        city = anjuke_city_map[city]
        column = ['shape', 'area', 'floor', 'part', 'direction', 'trans', 'price', 'name', 'location', 'title']
        data_all = []
        for i in range(page_no):
            print('get data from page %s' % (i+1))
            data_list, msg = anjuke_agent.get_rent(city, i+1)
            data_all.extend(data_list)
        df = pd.DataFrame(data_all, columns=column)
        return df, ''
    else:
        return None, '暂不支持该城市'


# encoding: utf-8

from .economy_interface import *

__all__ = ['disp_result',
           'get_house_price_index',
           'get_cpi', 'get_region_cpi',
           'get_ppi', 'get_region_ppi',
           'get_gdp', 'get_region_gdp', 'get_gdp_q2q',
           'get_gdp_y', 'get_region_gdp_y',
           'get_M0_M1_M2', 'get_fiscal_revenue', 'get_fiscal_expend',
           'get_manufacturing_pmi', 'get_non_manufacturing_pmi','get_pmi',
           'get_import_export', 'get_fdi',
           'get_retail_sales', 'get_online_retail_sales',
           'get_realestate_investment', 'get_region_realestate_investment',
           'get_fixed_asset_investment', 'get_region_fixed_asset_investment'
           'get_indicator_map', 'get_region_map', 'get_city_map',
           'get_population_size_y', 'get_population_structure_y',
           ]




# encoding: utf-8

from .nhindex_agent import NHIndexAgent

nh_agent = NHIndexAgent()

def get_index_list():
    return nh_agent.get_index_list()

def get_index_daily(index_code):
    return nh_agent.get_index_daily(index_code)

def get_index_snapshot():
    return nh_agent.get_index_snapshot()

def get_index_weight():
    return nh_agent.get_index_weight()




# encoding: utf-8
from opendatatools import stock
import unittest


class TestStock(unittest.TestCase):
    def test_get_index_list(self):
        index_list = stock.get_index_list('SH')
        assert (len(index_list) > 0)

        index_list = stock.get_index_list('SZ')
        assert (len(index_list) > 0)

    def test_get_index_component(self):
        stock_list = stock.get_index_component('000001.SH')
        assert (len(stock_list) > 0)

        stock_list = stock.get_index_component('399001.SZ')
        assert (len(stock_list) > 0)

    def test_get_rzrq_info(self):
        df_total, df_detail = stock.get_rzrq_info(market='SH', date='2020-12-24')
        assert (len(df_total) == 1)
        assert (len(df_detail) > 100)

        df_total, df_detail = stock.get_rzrq_info(market='SZ', date='2020-12-24')
        assert (len(df_total) == 1)
        assert (len(df_detail) > 100)

    def test_get_market_money_flow(self):
        df_total, _ = stock.get_hist_money_flow_market()
        assert (len(df_total) > 0)

    def test_get_trade_detail(self):
        df_total, _ = stock.get_trade_detail(symbol='000001.SZ', trade_date='2020-12-24')
        assert (len(df_total) > 0)



# encoding: utf-8

from .usstock_agent import USStockAgent

usstock_agent = USStockAgent()

def set_proxies(proxies):
    return usstock_agent.set_proxies(proxies)

def get_symbols():
    return usstock_agent.get_symbols()

def get_daily(symbol):
    return usstock_agent.get_daily(symbol)

def get_dividend(symbol):
    return usstock_agent.get_dividend(symbol)

def get_split(symbol):
    return usstock_agent.get_split(symbol)


# encoding: utf-8

import datetime

from .hsgt_agent import EastMoneyAgent
from .hkex_agent import HKExAgent

eastmoney_agent = EastMoneyAgent()
hkex_agent = HKExAgent()

def set_proxies(proxies):
    hkex_agent.set_proxies(proxies)
    eastmoney_agent.set_proxies(proxies)

# 获取陆港通 北向 持股 情况
# 参数：市场SH/SZ，日期YYYY-MM-DD
def get_lgt_share(market = 'SH', date = None):
    if date is None:
        i = 0
        while i < 5:
            pub_time = datetime.datetime.now() - datetime.timedelta(days=i)
            date = datetime.datetime.strftime(pub_time, "%Y-%m-%d")
            df = hkex_agent.get_lgt_share(market, date)
            if len(df) > 0:
                return df
            i = i + 1
        return None

    else:
        return hkex_agent.get_lgt_share(market, date)

# 单位：亿元
def get_realtime_moneyflow():
    return eastmoney_agent.get_realtime_moneyflow()

# HSMoney : 沪市北向资金  SSMoney : 深市北向资金  NorthMoney : 北向资金汇总
# GGHSMoney : 港股沪市资金  GGSSMoney : 港股深市资金  SouthSumMoney ：南向资金汇总
# 单位：百万元
def get_hist_moneyflow():
    return eastmoney_agent.get_hist_moneyflow()

# markettype: 1 沪股通 2 港股通（沪）3 深股通 4 港股通（深市）
# DRZJLR: 当日资金流入 DRYE：当日余额 LSZJLR：历史资金累计流入 DRCJJME：当日成交净买额 MRCJE：买入成交额 MCCJE：卖出成交额
# 其他字段可以忽略
# 单位：百万元（百万港元 - 涉及港股市场的成交和净买卖金额，单位都是百万港币）
def get_his_tradestat(markettype):
    return eastmoney_agent.get_his_tradestat(markettype)

def get_ah_compare():
    return eastmoney_agent.get_ah_compare()


# encoding: utf-8

from .spot_agent import SpotAgent

spot_agent = SpotAgent()

def get_commodity_spot_indicator():
    return spot_agent.get_commodity_spot_indicator()

def get_commodity_spot_indicator_data(indicator_id):
    return spot_agent.get_commodity_spot_indicator_data(indicator_id)





import json
import pandas as pd
from bs4 import BeautifulSoup
import datetime

from opendatatools.common import RestAgent

class WeiboAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)
        self.session.headers['User-Agent'] = "Mozilla/5.0 (Linux; Android 6.0; " \
                                               "Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Mobile Safari/537.36"
        self.session.headers['Referer'] = 'http://data.weibo.com/index/newindex'

    def _get_items(self, word):
        url = 'http://data.weibo.com/index/ajax/newindex/searchword'
        data = { "word": word }

        response = self.do_request(url, data)
        if response is None:
            return None

        jsonobj = json.loads(response)
        rsp = jsonobj['html']

        soup = BeautifulSoup(rsp, "html5lib")
        lis = soup.find_all('li')

        result = {}
        for li in lis:
            wid = li['wid']
            keyword = li['word']
            result[keyword] = wid

        return result

    # 1hour, 1day, 1month, 3month
    def _get_index_data(self, wid, type):
        url = 'http://data.weibo.com/index/ajax/newindex/getchartdata'
        data = {
            "wid": wid,
            "dateGroup": type,
        }

        response = self.do_request(url, data)
        if response is None:
            return None

        jsonobj = json.loads(response)
        data = {
            'index': jsonobj['data'][0]['trend']['x'],
            'value': jsonobj['data'][0]['trend']['s'],
        }

        df = pd.DataFrame(data)
        return df

    def _process_index(self, index):
        now = datetime.datetime.now()
        curr_year  = now.year
        curr_date = "%04d%02d%02d" % (now.year, now.month, now.day)
        if '月' in index:
            tmp = index.replace('日', '').split('月')
            date = "%04d%02d%02d" % (curr_year, int(tmp[0]), int(tmp[1]))
            if date > curr_date:
                date = "%04d%02d%02d" % (curr_year-1, int(tmp[0]), int(tmp[1]))

            return date

        return index

    # 1hour, 1day, 1month, 3month
    def get_weibo_index(self, word, type):
        dict_keyword = self._get_items(word)
        if dict_keyword is None:
            return None, '获取数据失败'

        df_list = []
        for keyword, wid in dict_keyword.items():
            df = self._get_index_data(wid, type)
            if df is not None:
                df.columns = ['index', keyword]
                df['index'] = df['index'].apply(lambda x : self._process_index(x))
                df.set_index('index', inplace=True)
                df_list.append(df)

        if len(df_list) > 0:
            return pd.concat(df_list, axis=1), ''

        return None, ''




# encoding: UTF-8

from urllib.parse import urlencode
from bs4 import BeautifulSoup
import datetime
import time
from opendatatools.common import RestAgent
import pandas as pd
import json
import math
import random


class AMACAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)
        self.add_headers({  # 请求头
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'Accept-Encoding': 'gzip, deflate',
            'Accept-Language': 'zh-CN,zh;q=0.9',
            'Cache-Control': 'max-age=0',
            'Connection': 'keep-alive',
            'Content-Length': '2',
            'Content-Type': 'application/json',
            'Host': 'gs.amac.org.cn',
            'Origin': 'http://gs.amac.org.cn',
            'Referer': 'http://gs.amac.org.cn/amac-infodisc/res/pof/fund/index.html',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.102 Safari/537.36',
            'X-Requested-With': 'XMLHttpRequest'
        })

        self.fund_list_url = 'http://gs.amac.org.cn/amac-infodisc/api/pof/fund?'
        self.fund_base_url = 'http://gs.amac.org.cn/amac-infodisc/res/pof/fund/'

        self.manager_list_url = 'http://gs.amac.org.cn/amac-infodisc/api/pof/manager?'  # 请求 url 所携带参数的前面部分
        self.manager_base_url = 'http://gs.amac.org.cn/amac-infodisc/res/pof/manager/'  # 基金 url 前面的部分

        self.company_title_list = [
                '私募基金管理人名称',
                '管理人详细信息网址',
                '管理人类型',
                '成立时间',
                '备案时间',
                '机构诚信信息',
                '基金管理人全称(中文)',
                '基金管理人全称(英文)',
                '登记编号',
                '组织机构代码',
                '登记时间',
                '成立时间',
                '注册地址',
                '办公地址',
                '注册资本(万元)(人民币)',
                '实缴资本(万元)(人民币)',
                '企业性质',
                '注册资本实缴比例',
                '机构类型',
                '业务类型',
                '全职员工人数',
                '取得基金从业人数',
                '机构网址',
            ]
        self.fund_title_list = [
            '基金名称',
            '基金编号',
            '成立时间',
            '备案时间',
            '基金备案阶段',
            '基金类型',
            '币种',
            '基金管理人名称',
            '管理类型',
            '托管人名称',
            '运作状态',
            '基金信息最后更新时间',
            '基金协会特别提示（针对基金）'
        ]

    def get_total_record(self, list_url):
        params = {
            'rand': random.random(),
            'page': 1,
            'size': 10,
        }
        url = list_url + urlencode(params)
        resp = self.do_request(url, json={}, method="POST")
        result = json.loads(resp)
        return result['totalElements']

    def get_page(self, list_url, page):
        '''爬取第xx页信息'''
        # url 携带参数，设置了每页显示 100 条信息
        params = {
            'rand': 0.3248183083707361,
            'page': page,
            'size': 1000,
        }
        url = list_url + urlencode(params)
        resp = self.do_request(url, json={}, method="POST")
        return json.loads(resp)

    def parse_fund_page(self, r_json):
        if r_json:
            items = r_json.get('content')
            for item in items:
                info = {}
                info['id'] = item.get('id')
                info['基金名称'] = item.get('fundName')
                info['基金详细信息网址'] = self.fund_base_url + item.get('id') + ".html"
                info['基金状态'] = item.get('workingState')
                info['私募基金管理人名称'] = item.get('managerName')
                info['管理人类型'] = item.get('managerType')
                establishDate = item.get('establishDate')
                info['成立时间'] = str(datetime.datetime.fromtimestamp(
                    establishDate / 1000).date()) if establishDate else ''  # 成立时间有可能为空，防止这种情况而报错
                putOnRecordDate = item.get('putOnRecordDate')
                info['备案时间'] = str(
                    datetime.datetime.fromtimestamp(putOnRecordDate / 1000).date()) if putOnRecordDate else ''
                yield info

    def parse_manager_page(self, r_json):
        if r_json:
            items = r_json.get('content')
            for item in items:
                info = {}
                info['id'] = item.get('id')
                info['私募基金管理人名称'] = item.get('managerName')
                info['管理人详细信息网址'] = self.manager_base_url + item.get('id') + ".html"
                info['管理人类型'] = item.get('primaryInvestType')
                establishDate = item.get('establishDate')
                info['成立时间'] = str(datetime.datetime.fromtimestamp(
                    establishDate / 1000).date()) if establishDate else ''  # 成立时间有可能为空，防止这种情况而报错
                registerDate = item.get('registerDate')
                info['备案时间'] = str(
                    datetime.datetime.fromtimestamp(registerDate / 1000).date()) if registerDate else ''
                yield info

    def get_detail(self, url):
        resp = self.do_request(url, method="GET", encoding="utf-8")
        return resp

    def parse_manager_detail(self, html):
        soup = BeautifulSoup(html, "html5lib")
        tables = soup.find_all('table')
        info = {}
        for table in tables:
            if table.has_attr("class") and "table-info" in table['class']:
                rows = table.findAll('tr')
                for row in rows:
                    cols = row.findAll('td')
                    if len(cols) >= 2:
                        title = cols[0].text
                        content = cols[1].text
                        title = title.replace(":", "")
                        content = content.replace("\n", "")
                        content = content.strip()
                        if title in self.company_title_list:
                            info[title] = content

                    if len(cols) >= 4:
                        title = cols[2].text
                        content = cols[3].text
                        title = title.replace(":", "")
                        content = content.replace("\n", "")
                        content = content.strip()
                        if title in self.company_title_list:
                            info[title] = content

        return info

    def parse_fund_detail(self, html):
        soup = BeautifulSoup(html, "html5lib")
        tables = soup.find_all('table')
        info = {}
        for table in tables:
            if table.has_attr("class") and "table-info" in table['class']:
                rows = table.findAll('tr')
                for row in rows:
                    cols = row.findAll('td')
                    if len(cols) >= 2:
                        title = cols[0].text
                        content = cols[1].text
                        title = title.replace(":", "")
                        content = content.replace("\n", "")
                        content = content.strip()
                        if title in self.fund_title_list:
                            info[title] = content

                    if len(cols) >= 4:
                        title = cols[2].text
                        content = cols[3].text
                        title = title.replace(":", "")
                        content = content.replace("\n", "")
                        content = content.strip()
                        if title in self.fund_title_list:
                            info[title] = content

        return info

    def get_company_list(self):
        total_record = self.get_total_record(self.manager_list_url)
        total_page = math.ceil(total_record / 1000)
        print(total_record, total_page)
        lis_json = []
        for page in range(1, total_page):
            print("page=", page)
            r_json = self.get_page(self.manager_list_url, page)
            results = self.parse_manager_page(r_json)
            for result in results:
                lis_json.append(result)

        return pd.DataFrame(lis_json)

    def get_company_detail(self, company_id):
        url = self.manager_base_url + company_id + ".html"
        html = self.get_detail(url)
        info = self.parse_manager_detail(html)
        return info

    def get_fund_list(self):
        total_record = self.get_total_record(self.fund_list_url)
        total_page = math.ceil(total_record / 1000)
        print(total_record, total_page)
        lis_json = []
        for page in range(1, total_page):
            print("page=", page)
            r_json = self.get_page(self.fund_list_url, page)
            results = self.parse_fund_page(r_json)
            for result in results:
                lis_json.append(result)

        return pd.DataFrame(lis_json)

    def get_fund_detail(self, fund_id):
        url = self.fund_base_url + fund_id + ".html"
        html = self.get_detail(url)
        info = self.parse_fund_detail(html)
        return info


if __name__ == '__main__':
    agent = AMACAgent()
    #df = agent.get_company_list()
    #print(df)

    #result = agent.get_company_detail("101000004390")
    #print(result)

    #df = agent.get_fund_list()
    #print(df)

    result = agent.get_fund_detail('351000130305')
    print(result)




from opendatatools.common import RestAgent
import pandas as pd
import json

top_items_map = {
    '中国宏观' : "402273",
    '行业经济' : "771263",
    '国际宏观' : "1138921",
    '特色数据' : "632815",
    '市场行情' : 'RRP1349982',
    '公司数据' : 'RRP1',
}

class RoboAgent(RestAgent):

    def __init__(self):
        RestAgent.__init__(self)
        self.token = ""

    def _extract_result(self, response):
        jsonobj = json.loads(response)
        code = None
        message = None
        if "code" in jsonobj:
            code = jsonobj['code']

        if "message" in jsonobj:
            message = jsonobj['message']

        return code, message

    def _extract_content(self, response, key='content'):
        jsonobj = json.loads(response)

        if key in jsonobj:
            return jsonobj[key]

        return None

    def login(self, username, password):
        url = "https://app.datayes.com/server/usermaster/authenticate/v1.json"
        param = {
            'x-requested-with': 'XMLHttpRequest',
            'username' : username,
            'password' : password,
        }
        '''
        {"code":0,"message":"Success","content":{"result":"FAIL","userId":0,"principalName":"9957@wmcloud.com","tenantId":0,"accountId":0}}
        {"code":0,"message":"Success","content":{"result":"SUCCESS","userId":9957,"principalName":"9957@wmcloud.com","tenantId":0,"token":{"tokenString":"09FB012D0072D5E368F5BAB8B54C51B1","type":"WEB","expiry":1533996916631,"expired":false},"redirectUrl":"https://app.wmcloud.com/cloud-portal/#/portal","accountId":18975}}
       '''
        response = self.do_request(url, param=param, method='POST')
        if response is None:
            return None, '获取数据失败'

        code, message = self._extract_result(response)
        if code is None or code != 0:
            return False, "登录失败:" + message

        content = self._extract_content(response)
        if content is None:
            return False, "登录失败，没有返回content"

        result = content['result']
        if result == "FAIL":
            return False, "登录失败"

        tokenString = content['token']['tokenString']
        self.token = tokenString

        return True, "登录成功"

    def get_top_items(self):
        return top_items_map

    def get_sub_items(self, itemid):
        url = 'https://gw.datayes.com/rrp/web/supervisor/macro/%s' % itemid
        response = self.do_request(url)

        if response is None:
            return None, "获取数据失败"

        code, message = self._extract_result(response)
        if code is None or code != 1:
            return None, "获取数据失败:" + message

        data = self._extract_content(response, 'data')
        if data is None:
            return None, "获取数据失败，没有返回data"

        df = pd.DataFrame(data['childData'])
        return df, ""

    def get_series(self, seriesid):
        url = "https://gw.datayes.com/rrp/web/dataCenter/indic/%s?compare=false" % seriesid
        response = self.do_request(url, method='GET', encoding='gzip')
        if response is None:
            return None, None, "获取数据失败"

        code, message = self._extract_result(response)
        if code is None or code != 1:
            return None, None, "获取数据失败:" + message

        data = self._extract_content(response, 'data')
        if data is None:
            return None, None, "获取数据失败，没有返回data"

        df_data = pd.DataFrame(data['data'])
        info   = data['indic']
        return df_data, info, ""




# -*- coding:utf-8 -*-

import datetime
import calendar

lastday_map = {}


def get_current_day(format="%Y-%m-%d"):
    curr_date = datetime.datetime.now()
    return datetime.datetime.strftime(curr_date, format)


def date_convert(date, format, target_format):
    return datetime.datetime.strftime(datetime.datetime.strptime(date, format), target_format)


def get_month_firstday_and_lastday(year=None, month=None):
    """
    :param year: 年份，默认是本年，可传int或str类型
    :param month: 月份，默认是本月，可传int或str类型
    :return: firstDay: 当月的第一天，datetime.date类型
              lastDay: 当月的最后一天，datetime.date类型
    """
    if year:
        year = int(year)
    else:
        year = datetime.date.today().year

    if month:
        month = int(month)
    else:
        month = datetime.date.today().month

    # 获取当月第一天的星期和当月的总天数
    firstDayWeekDay, monthRange = calendar.monthrange(year, month)

    # 获取当月的第一天
    firstDay = datetime.date(year=year, month=month, day=1)
    lastDay = datetime.date(year=year, month=month, day=monthRange)

    return firstDay, lastDay


def get_month_lastday(datestr, format="%Y-%m-%d"):
    if datestr in lastday_map:
        return lastday_map[datestr]

    date = datetime.datetime.strptime(datestr, format)
    year = date.year
    month = date.month
    firstDay, lastDay = get_month_firstday_and_lastday(year, month)
    result = datetime.datetime.strftime(lastDay, format)
    lastday_map[datestr] = result
    return result


def get_target_date2(date, span, format="%Y-%m-%d"):
    curr_date = datetime.datetime.strptime(date, format)
    span_days = datetime.timedelta(days=span)
    target = curr_date + span_days
    return datetime.datetime.strftime(target, format)


def get_target_date(span, format="%Y-%m-%d"):
    today = datetime.date.today()
    span_days = datetime.timedelta(days=span)
    target = today + span_days
    return datetime.datetime.strftime(target, format)


def split_date(datestr, format="%Y-%m-%d"):
    date = datetime.datetime.strptime(datestr, format)
    return date.year, date.month, date.day


if __name__ == '__main__':
    print(get_target_date(0))
    print(get_target_date(10))
    print(get_target_date(-1))



# encoding: utf-8

from .wscn_agent import XuangubaoAgent

xgb_agent = XuangubaoAgent()


def login(username, password):
    return xgb_agent.login(username, password)

def get_xuangubao_theme():
    return xgb_agent.get_theme()

def get_xuangubao_theme_stock(tid):
    return xgb_agent.get_theme_stock(tid)

def get_xuangubao_theme_kline(tid):
    return xgb_agent.get_theme_kline(tid)

def get_xuangubao_theme_event(tid):
    return xgb_agent.get_theme_event(tid)



# encoding: UTF-8

from setuptools import setup, find_packages
from opendatatools import __version__ as ver

setup(
    # Install data files specified in MANIFEST.in file.
    include_package_data=True,
    # package_data={'': ['*.json', '*.css', '*.html']},
    # Package Information
    name='InvestOpenDataTools',
    url='https://github.com/suuuch/InvestOpenData',
    version=ver,
    license='Apache 2.0',
    # information
    description='Open source data tools Fork .',
    long_description="",
    keywords="data,crawler,free",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Financial and Insurance Industry",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: Chinese (Simplified)",
        "Natural Language :: English",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: Unix",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
    ],
    # install
    install_requires=[
        'beautifulsoup4==4.9.3',
        'bs4==0.0.1',
        'certifi==2020.12.5',
        'chardet==4.0.0',
        'demjson==2.2.4',
        'idna==2.10',
        'numpy==1.19.4',
        'pandas==1.1.5',
        'Pillow==8.0.1',
        'progressbar2==3.53.1',
        'pytesseract==0.3.7',
        'python-dateutil==2.8.1',
        'python-utils==2.4.0',
        'pytz==2020.5',
        'requests==2.25.1',
        'six==1.15.0',
        'soupsieve==2.1',
        'urllib3==1.26.2',
        'xlrd==2.0.1',

    ],
    packages=find_packages(),
    # author
    author='suuuch'
)



from .gaokao_agent import GaokaoAgent

gaokao_agent = GaokaoAgent()


def set_proxies(proxies):
    gaokao_agent.set_proxies(proxies)

def get_school_list(text):
    return gaokao_agent.get_school_list(text)

def get_school_baseinfo(school):
    return gaokao_agent.get_school_baseinfo(school)

def get_school_major(school):
    return gaokao_agent.get_school_major(school)

def get_school_score(school):
    return gaokao_agent.get_school_score(school)

def get_batch_score(province, subject):
    return gaokao_agent.get_batch_score(province, subject)



# encoding: utf-8

import json
import pandas as pd
import datetime
from bs4 import BeautifulSoup
from opendatatools.common import RestAgent

index_weight_2018 = \
'''
<html>
 <head></head>
 <body>
  <div class="detail-content">
   <table cellpadding="0" cellspacing="0">
    <colgroup>
     <col width="64" span="9" style="width:64px" />
    </colgroup>
    <tbody>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="72">品种名称</td>
      <td width="92">南华商品指数</td>
      <td width="90">南华工业品指数</td>
      <td width="79">南华金属指数</td>
      <td width="78">南华能化指数</td>
      <td width="77">南华农产品指数</td>
      <td width="85">南华贵金属指数</td>
      <td width="79">南华黑色指数</td>
      <td width="80">南华有色金属指数</td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">螺纹钢</td>
      <td width="90">11.26%</td>
      <td width="87">14.50%</td>
      <td width="78">25.00%</td>
      <td width="78"><br /></td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79" style="word-break: break-all;">38.11%</td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">铜</td>
      <td width="90">10.57%</td>
      <td width="87">13.71%</td>
      <td width="78">25.00%</td>
      <td width="78"><br /></td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80">51.56%</td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">原油</td>
      <td width="90">9.52%</td>
      <td width="87">10.73%</td>
      <td width="78"><br /></td>
      <td width="78">14.75%</td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">铁矿石</td>
      <td width="90">8.45%</td>
      <td width="87">10.53%</td>
      <td width="78">23.23%</td>
      <td width="78"><br /></td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79" style="word-break: break-all;">27.26%</td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">焦炭</td>
      <td width="90">7.23%</td>
      <td width="87">8.98%</td>
      <td width="78"><br /></td>
      <td width="78">15.96%</td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79" style="word-break: break-all;">23.24%</td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">天然橡胶</td>
      <td width="90">6.74%</td>
      <td width="87">9.15%</td>
      <td width="78"><br /></td>
      <td width="78">19.32%</td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">豆粕</td>
      <td width="90">4.88%</td>
      <td width="87"><br /></td>
      <td width="78"><br /></td>
      <td width="78"><br /></td>
      <td width="77">20.54%</td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">黄金</td>
      <td width="90">4.66%</td>
      <td width="87"><br /></td>
      <td width="78"><br /></td>
      <td width="78"><br /></td>
      <td width="77"><br /></td>
      <td width="85">55.80%</td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">白银</td>
      <td width="90">4.32%</td>
      <td width="87"><br /></td>
      <td width="78"><br /></td>
      <td width="78"><br /></td>
      <td width="77"><br /></td>
      <td width="85">44.20%</td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">焦煤</td>
      <td width="90">3.93%</td>
      <td width="87">4.56%</td>
      <td width="78"><br /></td>
      <td width="78">6.78%</td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79" style="word-break: break-all;">11.39%</td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">镍</td>
      <td width="90">3.65%</td>
      <td width="87">4.89%</td>
      <td width="78">9.84%</td>
      <td width="78"><br /></td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80">17.25%</td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">白糖</td>
      <td width="90">3.30%</td>
      <td width="87"><br /></td>
      <td width="78"><br /></td>
      <td width="78"><br /></td>
      <td width="77">13.91%</td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">聚乙烯</td>
      <td width="90">3.18%</td>
      <td width="87">4.02%</td>
      <td width="78"><br /></td>
      <td width="78">7.42%</td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">豆油</td>
      <td width="90">3.12%</td>
      <td width="87"><br /></td>
      <td width="78"><br /></td>
      <td width="78"><br /></td>
      <td width="77">13.16%</td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">锌</td>
      <td width="90">3.04%</td>
      <td width="87">4.00%</td>
      <td width="78">8.25%</td>
      <td width="78"><br /></td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80">14.66%</td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">PTA</td>
      <td width="90">3.01%</td>
      <td width="87">3.85%</td>
      <td width="78"><br /></td>
      <td width="78">7.25%</td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">棕榈油</td>
      <td width="90">2.76%</td>
      <td width="87"><br /></td>
      <td width="78"><br /></td>
      <td width="78"><br /></td>
      <td width="77">11.62%</td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">聚丙烯</td>
      <td width="90">2.38%</td>
      <td width="87">3.01%</td>
      <td width="78"><br /></td>
      <td width="78">5.51%</td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">甲醇</td>
      <td width="90">2.00%</td>
      <td width="87">2.42%</td>
      <td width="78"><br /></td>
      <td width="78">4.59%</td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">一号棉</td>
      <td width="90">2.00%</td>
      <td width="87"><br /></td>
      <td width="78"><br /></td>
      <td width="78"><br /></td>
      <td width="77">7.30%</td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">玻璃</td>
      <td width="90"><br /></td>
      <td width="87">2.00%</td>
      <td width="78"><br /></td>
      <td width="78">3.31%</td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">铝</td>
      <td width="90"><br /></td>
      <td width="87">3.65%</td>
      <td width="78">8.68%</td>
      <td width="78"><br /></td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80">16.53%</td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">石油沥青</td>
      <td width="90"><br /></td>
      <td width="87"><br /></td>
      <td width="78"><br /></td>
      <td width="78">2.98%</td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">动力煤</td>
      <td width="90"><br /></td>
      <td width="87"><br /></td>
      <td width="78"><br /></td>
      <td width="78">12.13%</td>
      <td width="77"><br /></td>
      <td width="85"><br /></td>
      <td width="79" style="word-break: break-all;"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" colspan="2" style="" width="68">黄大豆一号</td>
      <td width="87"><br /></td>
      <td width="78"><br /></td>
      <td width="78"><br /></td>
      <td width="77">11.01%</td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">玉米</td>
      <td width="90"><br /></td>
      <td width="87"><br /></td>
      <td width="78"><br /></td>
      <td width="78"><br /></td>
      <td width="77">10.63%</td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">菜籽油</td>
      <td width="90"><br /></td>
      <td width="87"><br /></td>
      <td width="78"><br /></td>
      <td width="78"><br /></td>
      <td width="77">4.75%</td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">鸡蛋</td>
      <td width="90"><br /></td>
      <td width="87"><br /></td>
      <td width="78"><br /></td>
      <td width="78"><br /></td>
      <td width="77">4.45%</td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
     <tr height="19" style="height:19px">
      <td height="19" style="" width="38">玉米淀粉</td>
      <td width="90"><br /></td>
      <td width="87"><br /></td>
      <td width="78"><br /></td>
      <td width="78"><br /></td>
      <td width="77">2.63%</td>
      <td width="85"><br /></td>
      <td width="79"><br /></td>
      <td width="80"><br /></td>
     </tr>
    </tbody>
   </table>
  </div> 
 </body>
</html>
'''

class NHIndexAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    def get_index_list(self):
        url = 'http://www.nanhua.net/ianalysis/plate-variety.json'
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'
        data = json.loads(response)
        df = pd.DataFrame(data)
        return df, ''

    def get_index_daily(self, index_code):
        url = 'http://www.nanhua.net/ianalysis/varietyindex/index/%s.json' % index_code
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'
        data = json.loads(response)
        df = pd.DataFrame(data)
        df.columns = ['date', 'close']
        df['date'] = df['date'].apply(lambda x : datetime.datetime.fromtimestamp(int(x)/1000))
        return df, ''

    def get_index_snapshot(self):
        url = 'http://www.nanhua.net/index/jsons/latestindex.json'
        response = self.do_request(url)
        if response is None:
            return None, '获取数据失败'
        data = json.loads(response)
        df = pd.DataFrame(data)
        df['lastUpdate'] = df['lastUpdate'].apply(lambda x : datetime.datetime.strptime(x, '%Y%m%d%H%M%S'))
        return df, ''

    def _convert_percent(self, text):
        text = text.strip()
        if len(text) == 0:
            return 0.0

        return float(float(text.strip('%')))/100

    def get_index_weight(self):
        soup = BeautifulSoup(index_weight_2018, "html5lib")
        divs = soup.find_all('div')
        table = divs[0].table
        rows = table.findAll('tr')

        head = []
        data = []
        for row in rows:
            cols = row.findAll('td')
            if len(cols) == 9:

                if cols[0].text == '品种名称':
                    for i in range(1, 9):
                        head.append(cols[i].text)
                    continue

                item = {}
                item["product"] = cols[0].text
                for i in range(1, 9):
                    item['weight' + str(i)] = self._convert_percent(cols[i].text)
                data.append(item)

        df = pd.DataFrame(data)
        df.set_index('product', inplace=True)
        df.columns = head

        return df, ''




# encoding: utf-8

import requests
from PIL import Image
import pytesseract
import io
import pandas as pd
from bs4 import BeautifulSoup
from datetime import datetime
from opendatatools.common import RestAgent

dict_commodity_spot_indicator = {
    '65': '钢材指数',
    '61': '铁矿指数',
    '64': '焦炭指数',
    '1002': '煤炭指数',
    '1003': '水泥指数',
    '1100': 'FTZ指数',
    '118': '钢铁行业PMI指数',
    '119': '钢铁行业PMI生产指数',
    '120': '钢铁行业PMI新订单指数',
    '121': '钢铁行业PMI新出口订单指数',
    '122': '钢铁行业PMI产成品库存指数',
    '123': '钢铁行业PMI原材料库存指数',
    '74': '沪市终端线螺每周采购量监控',
    '72': '沪螺纹钢社会库存',
    '67': '国内螺纹钢社会库存量',
    '68': '国内线材社会库存量',
    '69': '国内主要城市热轧卷板库存',
    '70': '国内主要城市冷轧卷板库存',
    '73': '国内主要城市中厚板库存',
    '117': '全国主要钢材品种库存总量',
    '108': '热轧价格走势',
    '109': '冷轧价格走势',
    '110': '中板价格走势',
    '111': '型材价格走势',
    '127': '沪二级螺纹钢价格走势',
    '99': '重点企业粗钢日均产量（旬报）',
    '124': '重点企业钢材库存量（旬报）',
    '159': '国内月度粗钢日均产量',
    '35': '国内月度粗钢产量',
    '88': '国内月度钢材产量',
    '40': '国内月度螺纹钢产量',
    '41': '国内月度线材产量',
    '114': '国内月度热轧板卷产量',
    '115': '国内月度冷轧板卷产量',
    '116': '国内月度中厚板产量',
    '177': '国内月度生铁产量',
    '37': '国内月度焦炭产量',
    '36': '国内月度铁矿石原矿产量',
    '42': '国内月度铁矿石进口量',
    '38': '国内月度钢材出口量',
    '39': '国内月度钢材进口量',
    '43': '国内铁矿石港口存量',
    '161': '唐山地区钢坯库存量',
    '100': '印度矿港口库存',
    '77': '波罗的海干散货指数（BDI）',
    '78': '废钢价格走势',
    '79': '钢坯价格走势',
    '178': '钢材成本指数',
    '93': '铁矿石进口月度均价',
    '94': '巴西图巴朗-北仑铁矿海运价',
    '95': '西澳-北仑铁矿海运价',
    '1006': '澳大利亚粉矿价格（56.5%，日照港）',
    '106': '澳大利亚粉矿价格(61.5%青岛港，元/吨）',
    '107': '巴西粉矿价格（ 65% 日照港，元/吨）',
    '125': '62%铁矿石指数',
    '126': '63.5%印度粉矿外盘报价',
    '162': '全球粗钢月度产量（万吨）',
    '163': '全球粗钢日均产量（万吨）',
    '164': '全球粗钢产能利用率（%）',
}

class SpotAgent(RestAgent):
    def __init__(self):
        RestAgent.__init__(self)

    @staticmethod
    def clear_text(text):
        return text.replace('\n', '').strip()

    def get_captcha(self):
        url = 'http://www.96369.net/Other/ValidateCode.aspx'
        response = self.do_request(url, method='GET', type='binary')
        img = Image.open(io.BytesIO(response))
        img = img.convert('L')

        # 二值化
        threshold = 160
        table = []
        for i in range(256):
            if i < threshold:
                table.append(0)
            else:
                table.append(1)

        # 由于都是数字
        # 对于识别成字母的 采用该表进行修正
        rep = {'O': '0',
               'I': '1', 'L': '1',
               'Z': '2',
               'S': '8'
               }

        img = img.point(table, '1')
        img = img.convert('RGBA')
        rsp = pytesseract.image_to_string(img, config='-psm 6')
        try:
            answer = eval(rsp)
            return answer, self.get_cookies()
        except:
            return None, None

    def get_commodity_spot_indicator(self):
        df = pd.DataFrame.from_dict(dict_commodity_spot_indicator, orient='index')
        df.columns=['indicator_name']
        df = df.rename_axis('indicator_id').reset_index()
        return df

    def get_commodity_spot_indicator_data(self, indicator_id):
        url = 'http://www.96369.net/indices/%s' % indicator_id
        captcha, cookies = self.get_captcha()
        end_date = datetime.now().strftime('%Y-%m-%d')
        data = {
            'txtStartTime': '2000-01-01',
            'txtEndTime': end_date,
            'txtyzcode': captcha
        }
        response = self.do_request(url, param=data, cookies=cookies)

        if response is None:
            return None, '获取数据失败'

        soup = BeautifulSoup(response, "html5lib")
        divs = soup.find_all('div')

        data = []
        for div in divs:
            if div.has_attr('class') and 'wll-commodity' in div['class']:
                tables = div.find_all('table')

                for table in tables:
                    if table.has_attr('class') and 'mod_tab' in table['class']:
                        rows = table.findAll('tr')
                        for row in rows:
                            cols = row.findAll('td')
                            if len(cols) == 4:
                                date = SpotAgent.clear_text(cols[0].text)
                                value = SpotAgent.clear_text(cols[1].text)
                                change = SpotAgent.clear_text(cols[2].text)
                                chg_pct = SpotAgent.clear_text(cols[3].text)

                                data.append({
                                    "date": date,
                                    "value": value,
                                    "change": change,
                                    "chg_pct": chg_pct,
                                })

        return pd.DataFrame(data), ''




# encoding: utf-8

from .futures_agent import SHFAgent, DCEAgent, CZCAgent, CFEAgent, SinaFuturesAgent
from opendatatools.common import get_target_date, date_convert

shf_agent = SHFAgent()
dce_agent = DCEAgent()
czc_agent = CZCAgent()
cfe_agent = CFEAgent()
sina_agent = SinaFuturesAgent()

def get_trade_rank(market = 'SHF', date = None):
    if date is None:
        date = get_target_date(-1, "%Y-%m-%d")

    if market == 'SHF':
        return shf_agent.get_trade_rank(date)

    if market == 'DCE':
        return dce_agent.get_trade_rank(date)

    if market == 'CZC':
        return czc_agent.get_trade_rank(date)

    if market == "CFE":
        return cfe_agent.get_trade_rank(date)

    return None, '不支持的市场类型'

def get_quote(codes):
    return sina_agent.get_quote(codes)

def get_kline(type, code):
    return sina_agent.get_kline(type, code)